/* RISC-V opcode list
   Copyright (C) 2011-2021 Free Software Foundation, Inc.

   Contributed by Andrew Waterman (andrew@sifive.com).
   Based on MIPS target.

   This file is part of the GNU opcodes library.

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   It is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING3. If not,
   see <http://www.gnu.org/licenses/>.  */

#include "sysdep.h"
#include "opcode/riscv.h"
#include <stdio.h>

/* Register names used by gas and objdump.  */

const char * const riscv_gpr_names_numeric[NGPR] =
{
  "x0",   "x1",   "x2",   "x3",   "x4",   "x5",   "x6",   "x7",
  "x8",   "x9",   "x10",  "x11",  "x12",  "x13",  "x14",  "x15",
  "x16",  "x17",  "x18",  "x19",  "x20",  "x21",  "x22",  "x23",
  "x24",  "x25",  "x26",  "x27",  "x28",  "x29",  "x30",  "x31"
};

const char * const riscv_gpr_names_abi[NGPR] = {
  "zero", "ra", "sp",  "gp",  "tp", "t0",  "t1",  "t2",
  "s0",   "s1", "a0",  "a1",  "a2", "a3",  "a4",  "a5",
  "a6",   "a7", "s2",  "s3",  "s4", "s5",  "s6",  "s7",
  "s8",   "s9", "s10", "s11", "t3", "t4",  "t5",  "t6"
};

const char * const riscv_fpr_names_numeric[NFPR] =
{
  "f0",   "f1",   "f2",   "f3",   "f4",   "f5",   "f6",   "f7",
  "f8",   "f9",   "f10",  "f11",  "f12",  "f13",  "f14",  "f15",
  "f16",  "f17",  "f18",  "f19",  "f20",  "f21",  "f22",  "f23",
  "f24",  "f25",  "f26",  "f27",  "f28",  "f29",  "f30",  "f31"
};

const char * const riscv_fpr_names_abi[NFPR] = {
  "ft0", "ft1", "ft2",  "ft3",  "ft4", "ft5", "ft6",  "ft7",
  "fs0", "fs1", "fa0",  "fa1",  "fa2", "fa3", "fa4",  "fa5",
  "fa6", "fa7", "fs2",  "fs3",  "fs4", "fs5", "fs6",  "fs7",
  "fs8", "fs9", "fs10", "fs11", "ft8", "ft9", "ft10", "ft11"
};

/* The order of overloaded instructions matters.  Label arguments and
   register arguments look the same. Instructions that can have either
   for arguments must apear in the correct order in this table for the
   assembler to pick the right one. In other words, entries with
   immediate operands must apear after the same instruction with
   registers.

   Because of the lookup algorithm used, entries with the same opcode
   name must be contiguous.  */

#define MASK_RS1 (OP_MASK_RS1 << OP_SH_RS1)
#define MASK_RS2 (OP_MASK_RS2 << OP_SH_RS2)
#define MASK_RD (OP_MASK_RD << OP_SH_RD)
#define MASK_CRS2 (OP_MASK_CRS2 << OP_SH_CRS2)
#define MASK_IMM ENCODE_ITYPE_IMM (-1U)
#define MASK_RVC_IMM ENCODE_RVC_IMM (-1U)
#define MASK_UIMM ENCODE_UTYPE_IMM (-1U)
#define MASK_RM (OP_MASK_RM << OP_SH_RM)
#define MASK_PRED (OP_MASK_PRED << OP_SH_PRED)
#define MASK_SUCC (OP_MASK_SUCC << OP_SH_SUCC)
#define MASK_AQ (OP_MASK_AQ << OP_SH_AQ)
#define MASK_RL (OP_MASK_RL << OP_SH_RL)
#define MASK_AQRL (MASK_AQ | MASK_RL)

static int
match_opcode (const struct riscv_opcode *op, insn_t insn)
{
  return ((insn ^ op->match) & op->mask) == 0;
}

static int
match_never (const struct riscv_opcode *op ATTRIBUTE_UNUSED,
	     insn_t insn ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
match_rs1_eq_rs2 (const struct riscv_opcode *op, insn_t insn)
{
  int rs1 = (insn & MASK_RS1) >> OP_SH_RS1;
  int rs2 = (insn & MASK_RS2) >> OP_SH_RS2;
  return match_opcode (op, insn) && rs1 == rs2;
}

static int
match_rd_nonzero (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && ((insn & MASK_RD) != 0);
}

static int
match_c_add (const struct riscv_opcode *op, insn_t insn)
{
  return match_rd_nonzero (op, insn) && ((insn & MASK_CRS2) != 0);
}

/* We don't allow mv zero,X to become a c.mv hint, so we need a separate
   matching function for this.  */

static int
match_c_add_with_hint (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && ((insn & MASK_CRS2) != 0);
}

static int
match_c_nop (const struct riscv_opcode *op, insn_t insn)
{
  return (match_opcode (op, insn)
	  && (((insn & MASK_RD) >> OP_SH_RD) == 0));
}

static int
match_c_addi16sp (const struct riscv_opcode *op, insn_t insn)
{
  return (match_opcode (op, insn)
	  && (((insn & MASK_RD) >> OP_SH_RD) == 2)
	  && EXTRACT_RVC_ADDI16SP_IMM (insn) != 0);
}

static int
match_c_lui (const struct riscv_opcode *op, insn_t insn)
{
  return (match_rd_nonzero (op, insn)
	  && (((insn & MASK_RD) >> OP_SH_RD) != 2)
	  && EXTRACT_RVC_LUI_IMM (insn) != 0);
}

/* We don't allow lui zero,X to become a c.lui hint, so we need a separate
   matching function for this.  */

static int
match_c_lui_with_hint (const struct riscv_opcode *op, insn_t insn)
{
  return (match_opcode (op, insn)
	  && (((insn & MASK_RD) >> OP_SH_RD) != 2)
	  && EXTRACT_RVC_LUI_IMM (insn) != 0);
}

static int
match_c_addi4spn (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && EXTRACT_RVC_ADDI4SPN_IMM (insn) != 0;
}

/* This requires a non-zero shift.  A zero rd is a hint, so is allowed.  */

static int
match_c_slli (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && EXTRACT_RVC_IMM (insn) != 0;
}

/* This requires a non-zero rd, and a non-zero shift.  */

static int
match_slli_as_c_slli (const struct riscv_opcode *op, insn_t insn)
{
  return match_rd_nonzero (op, insn) && EXTRACT_RVC_IMM (insn) != 0;
}

/* This requires a zero shift.  A zero rd is a hint, so is allowed.  */

static int
match_c_slli64 (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && EXTRACT_RVC_IMM (insn) == 0;
}

/* This is used for both srli and srai.  This requires a non-zero shift.
   A zero rd is not possible.  */

static int
match_srxi_as_c_srxi (const struct riscv_opcode *op, insn_t insn)
{
  return match_opcode (op, insn) && EXTRACT_RVC_IMM (insn) != 0;
}

const struct riscv_opcode riscv_opcodes[] =
{
/* name,     xlen, isa,   operands, match, mask, match_func, pinfo.  */
{"unimp",       0, INSN_CLASS_C,   "",  0, 0xffffU,  match_opcode, INSN_ALIAS },
{"unimp",       0, INSN_CLASS_I,   "",  MATCH_CSRRW | (CSR_CYCLE << OP_SH_CSR), 0xffffffffU,  match_opcode, 0 }, /* csrw cycle, x0 */
{"ebreak",      0, INSN_CLASS_C,   "",  MATCH_C_EBREAK, MASK_C_EBREAK, match_opcode, INSN_ALIAS },
{"ebreak",      0, INSN_CLASS_I,   "",    MATCH_EBREAK, MASK_EBREAK, match_opcode, 0 },
{"sbreak",      0, INSN_CLASS_C,   "",  MATCH_C_EBREAK, MASK_C_EBREAK, match_opcode, INSN_ALIAS },
{"sbreak",      0, INSN_CLASS_I,   "",    MATCH_EBREAK, MASK_EBREAK, match_opcode, INSN_ALIAS },
{"ret",         0, INSN_CLASS_C,   "",  MATCH_C_JR | (X_RA << OP_SH_RD), MASK_C_JR | MASK_RD, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"ret",         0, INSN_CLASS_I,   "",  MATCH_JALR | (X_RA << OP_SH_RS1), MASK_JALR | MASK_RD | MASK_RS1 | MASK_IMM, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"jr",          0, INSN_CLASS_C,   "d",  MATCH_C_JR, MASK_C_JR, match_rd_nonzero, INSN_ALIAS|INSN_BRANCH },
{"jr",          0, INSN_CLASS_I,   "s",  MATCH_JALR, MASK_JALR | MASK_RD | MASK_IMM, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"jr",          0, INSN_CLASS_I,   "o(s)",  MATCH_JALR, MASK_JALR | MASK_RD, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"jr",          0, INSN_CLASS_I,   "s,j",  MATCH_JALR, MASK_JALR | MASK_RD, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"jalr",        0, INSN_CLASS_C,   "d",  MATCH_C_JALR, MASK_C_JALR, match_rd_nonzero, INSN_ALIAS|INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "s",  MATCH_JALR | (X_RA << OP_SH_RD), MASK_JALR | MASK_RD | MASK_IMM, match_opcode, INSN_ALIAS|INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "o(s)",  MATCH_JALR | (X_RA << OP_SH_RD), MASK_JALR | MASK_RD, match_opcode, INSN_ALIAS|INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "s,j",  MATCH_JALR | (X_RA << OP_SH_RD), MASK_JALR | MASK_RD, match_opcode, INSN_ALIAS|INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "d,s",  MATCH_JALR, MASK_JALR | MASK_IMM, match_opcode, INSN_ALIAS|INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "d,o(s)",  MATCH_JALR, MASK_JALR, match_opcode, INSN_JSR },
{"jalr",        0, INSN_CLASS_I,   "d,s,j",  MATCH_JALR, MASK_JALR, match_opcode, INSN_JSR },
{"j",           0, INSN_CLASS_C,   "Ca",  MATCH_C_J, MASK_C_J, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"j",           0, INSN_CLASS_I,   "a",  MATCH_JAL, MASK_JAL | MASK_RD, match_opcode, INSN_ALIAS|INSN_BRANCH },
{"jal",         0, INSN_CLASS_I,   "d,a",  MATCH_JAL, MASK_JAL, match_opcode, INSN_JSR },
{"jal",        32, INSN_CLASS_C,   "Ca",  MATCH_C_JAL, MASK_C_JAL, match_opcode, INSN_ALIAS|INSN_JSR },
{"jal",         0, INSN_CLASS_I,   "a",  MATCH_JAL | (X_RA << OP_SH_RD), MASK_JAL | MASK_RD, match_opcode, INSN_ALIAS|INSN_JSR },
{"call",        0, INSN_CLASS_I,   "d,c", (X_T1 << OP_SH_RS1), (int) M_CALL,  match_never, INSN_MACRO },
{"call",        0, INSN_CLASS_I,   "c", (X_RA << OP_SH_RS1) | (X_RA << OP_SH_RD), (int) M_CALL,  match_never, INSN_MACRO },
{"tail",        0, INSN_CLASS_I,   "c", (X_T1 << OP_SH_RS1), (int) M_CALL,  match_never, INSN_MACRO },
{"jump",        0, INSN_CLASS_I,   "c,s", 0, (int) M_CALL,  match_never, INSN_MACRO },
{"nop",         0, INSN_CLASS_C,   "",  MATCH_C_ADDI, 0xffff, match_opcode, INSN_ALIAS },
{"nop",         0, INSN_CLASS_I,   "",         MATCH_ADDI, MASK_ADDI | MASK_RD | MASK_RS1 | MASK_IMM, match_opcode, INSN_ALIAS },
{"lui",         0, INSN_CLASS_C,   "d,Cu",  MATCH_C_LUI, MASK_C_LUI, match_c_lui, INSN_ALIAS },
{"lui",         0, INSN_CLASS_I,   "d,u",  MATCH_LUI, MASK_LUI, match_opcode, 0 },
{"li",          0, INSN_CLASS_C,   "d,Cv",  MATCH_C_LUI, MASK_C_LUI, match_c_lui, INSN_ALIAS },
{"li",          0, INSN_CLASS_C,   "d,Co",  MATCH_C_LI, MASK_C_LI, match_rd_nonzero, INSN_ALIAS },
{"li",          0, INSN_CLASS_I,   "d,j",      MATCH_ADDI, MASK_ADDI | MASK_RS1, match_opcode, INSN_ALIAS }, /* addi */
{"li",          0, INSN_CLASS_I,   "d,I",  0,    (int) M_LI,  match_never, INSN_MACRO },
{"mv",          0, INSN_CLASS_C,   "d,CV",  MATCH_C_MV, MASK_C_MV, match_c_add, INSN_ALIAS },
{"mv",          0, INSN_CLASS_I,   "d,s",  MATCH_ADDI, MASK_ADDI | MASK_IMM, match_opcode, INSN_ALIAS },
{"move",        0, INSN_CLASS_C,   "d,CV",  MATCH_C_MV, MASK_C_MV, match_c_add, INSN_ALIAS },
{"move",        0, INSN_CLASS_I,   "d,s",  MATCH_ADDI, MASK_ADDI | MASK_IMM, match_opcode, INSN_ALIAS },
{"sext.b",    0, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_SEXT_B, MASK_SEXT_B, match_opcode, 0 },
{"sext.b",      0, INSN_CLASS_I,   "d,s",  0, (int) M_SEXTB, match_never, INSN_MACRO },
{"sext.h",    0, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_SEXT_H, MASK_SEXT_H, match_opcode, 0 },
{"sext.h",      0, INSN_CLASS_I,   "d,s",  0, (int) M_SEXTH, match_never, INSN_MACRO },
{"zext.b",      0, INSN_CLASS_I,   "d,s",  MATCH_ANDI | ENCODE_ITYPE_IMM (255), MASK_ANDI | MASK_IMM, match_opcode, INSN_ALIAS },
{"zext.h",   32, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_PACK, MASK_PACK | MASK_RS2, match_opcode, 0 },
{"zext.h",   64, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_PACKW, MASK_PACKW | MASK_RS2, match_opcode, 0 },
{"zext.h",      0, INSN_CLASS_I,   "d,s",  0, (int) M_ZEXTH, match_never, INSN_MACRO },
{"andi",        0, INSN_CLASS_C,   "Cs,Cw,Co",  MATCH_C_ANDI, MASK_C_ANDI, match_opcode, INSN_ALIAS },
{"andi",        0, INSN_CLASS_I,   "d,s,j",  MATCH_ANDI, MASK_ANDI, match_opcode, 0 },
{"and",         0, INSN_CLASS_C,   "Cs,Cw,Ct",  MATCH_C_AND, MASK_C_AND, match_opcode, INSN_ALIAS },
{"and",         0, INSN_CLASS_C,   "Cs,Ct,Cw",  MATCH_C_AND, MASK_C_AND, match_opcode, INSN_ALIAS },
{"and",         0, INSN_CLASS_C,   "Cs,Cw,Co",  MATCH_C_ANDI, MASK_C_ANDI, match_opcode, INSN_ALIAS },
{"and",         0, INSN_CLASS_I,   "d,s,t",  MATCH_AND, MASK_AND, match_opcode, 0 },
{"and",         0, INSN_CLASS_I,   "d,s,j",  MATCH_ANDI, MASK_ANDI, match_opcode, INSN_ALIAS },
{"beqz",        0, INSN_CLASS_C,   "Cs,Cp",  MATCH_C_BEQZ, MASK_C_BEQZ, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"beqz",        0, INSN_CLASS_I,   "s,p",  MATCH_BEQ, MASK_BEQ | MASK_RS2, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"beq",         0, INSN_CLASS_C,   "Cs,Cz,Cp",  MATCH_C_BEQZ, MASK_C_BEQZ, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"beq",         0, INSN_CLASS_I,   "s,t,p",  MATCH_BEQ, MASK_BEQ, match_opcode, INSN_CONDBRANCH },
{"blez",        0, INSN_CLASS_I,   "t,p",  MATCH_BGE, MASK_BGE | MASK_RS1, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bgez",        0, INSN_CLASS_I,   "s,p",  MATCH_BGE, MASK_BGE | MASK_RS2, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bge",         0, INSN_CLASS_I,   "s,t,p",  MATCH_BGE, MASK_BGE, match_opcode, INSN_CONDBRANCH },
{"bgeu",        0, INSN_CLASS_I,   "s,t,p",  MATCH_BGEU, MASK_BGEU, match_opcode, INSN_CONDBRANCH },
{"ble",         0, INSN_CLASS_I,   "t,s,p",  MATCH_BGE, MASK_BGE, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bleu",        0, INSN_CLASS_I,   "t,s,p",  MATCH_BGEU, MASK_BGEU, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bltz",        0, INSN_CLASS_I,   "s,p",  MATCH_BLT, MASK_BLT | MASK_RS2, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bgtz",        0, INSN_CLASS_I,   "t,p",  MATCH_BLT, MASK_BLT | MASK_RS1, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"blt",         0, INSN_CLASS_I,   "s,t,p",  MATCH_BLT, MASK_BLT, match_opcode, INSN_CONDBRANCH },
{"bltu",        0, INSN_CLASS_I,   "s,t,p",  MATCH_BLTU, MASK_BLTU, match_opcode, INSN_CONDBRANCH },
{"bgt",         0, INSN_CLASS_I,   "t,s,p",  MATCH_BLT, MASK_BLT, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bgtu",        0, INSN_CLASS_I,   "t,s,p",  MATCH_BLTU, MASK_BLTU, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bnez",        0, INSN_CLASS_C,   "Cs,Cp",  MATCH_C_BNEZ, MASK_C_BNEZ, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bnez",        0, INSN_CLASS_I,   "s,p",  MATCH_BNE, MASK_BNE | MASK_RS2, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bne",         0, INSN_CLASS_C,   "Cs,Cz,Cp",  MATCH_C_BNEZ, MASK_C_BNEZ, match_opcode, INSN_ALIAS|INSN_CONDBRANCH },
{"bne",         0, INSN_CLASS_I,   "s,t,p",  MATCH_BNE, MASK_BNE, match_opcode, INSN_CONDBRANCH },
{"addi",        0, INSN_CLASS_C,   "Ct,Cc,CK", MATCH_C_ADDI4SPN, MASK_C_ADDI4SPN, match_c_addi4spn, INSN_ALIAS },
{"addi",        0, INSN_CLASS_C,   "d,CU,Cj",  MATCH_C_ADDI, MASK_C_ADDI, match_rd_nonzero, INSN_ALIAS },
{"addi",        0, INSN_CLASS_C,   "d,CU,z",    MATCH_C_NOP, MASK_C_ADDI | MASK_RVC_IMM, match_c_nop, INSN_ALIAS },
{"addi",        0, INSN_CLASS_C,   "Cc,Cc,CL", MATCH_C_ADDI16SP, MASK_C_ADDI16SP, match_c_addi16sp, INSN_ALIAS },
{"addi",        0, INSN_CLASS_C,   "d,Cz,Co",  MATCH_C_LI, MASK_C_LI, match_rd_nonzero, INSN_ALIAS },
{"addi",        0, INSN_CLASS_I,   "d,s,j",  MATCH_ADDI, MASK_ADDI, match_opcode, 0 },
{"add",         0, INSN_CLASS_C,   "d,CU,CV",  MATCH_C_ADD, MASK_C_ADD, match_c_add, INSN_ALIAS },
{"add",         0, INSN_CLASS_C,   "d,CV,CU",  MATCH_C_ADD, MASK_C_ADD, match_c_add, INSN_ALIAS },
{"add",         0, INSN_CLASS_C,   "d,CU,Co",  MATCH_C_ADDI, MASK_C_ADDI, match_rd_nonzero, INSN_ALIAS },
{"add",         0, INSN_CLASS_C,   "Ct,Cc,CK", MATCH_C_ADDI4SPN, MASK_C_ADDI4SPN, match_c_addi4spn, INSN_ALIAS },
{"add",         0, INSN_CLASS_C,   "Cc,Cc,CL", MATCH_C_ADDI16SP, MASK_C_ADDI16SP, match_c_addi16sp, INSN_ALIAS },
{"add",         0, INSN_CLASS_C,   "d,Cz,CV",  MATCH_C_MV, MASK_C_MV, match_c_add, INSN_ALIAS },
{"add",         0, INSN_CLASS_I,   "d,s,t",  MATCH_ADD, MASK_ADD, match_opcode, 0 },
/* This is used for TLS, where the fourth arg is %tprel_add, to get a reloc
   applied to an add instruction, for relaxation to use.  */
{"add",         0, INSN_CLASS_I,   "d,s,t,1",MATCH_ADD, MASK_ADD, match_opcode, 0 },
{"add",         0, INSN_CLASS_I,   "d,s,j",  MATCH_ADDI, MASK_ADDI, match_opcode, INSN_ALIAS },
{"la",          0, INSN_CLASS_I,   "d,B",  0,    (int) M_LA,  match_never, INSN_MACRO },
{"lla",         0, INSN_CLASS_I,   "d,B",  0,    (int) M_LLA,  match_never, INSN_MACRO },
{"la.tls.gd",   0, INSN_CLASS_I,   "d,A",  0,    (int) M_LA_TLS_GD,  match_never, INSN_MACRO },
{"la.tls.ie",   0, INSN_CLASS_I,   "d,A",  0,    (int) M_LA_TLS_IE,  match_never, INSN_MACRO },
{"neg",         0, INSN_CLASS_I,   "d,t",  MATCH_SUB, MASK_SUB | MASK_RS1, match_opcode, INSN_ALIAS }, /* sub 0 */
{"slli",        0, INSN_CLASS_C,   "d,CU,C>",  MATCH_C_SLLI, MASK_C_SLLI, match_slli_as_c_slli, INSN_ALIAS },
{"slli",        0, INSN_CLASS_I,   "d,s,>",   MATCH_SLLI, MASK_SLLI, match_opcode, 0 },
{"sll",         0, INSN_CLASS_C,   "d,CU,C>",  MATCH_C_SLLI, MASK_C_SLLI, match_slli_as_c_slli, INSN_ALIAS },
{"sll",         0, INSN_CLASS_I,   "d,s,t",   MATCH_SLL, MASK_SLL, match_opcode, 0 },
{"sll",         0, INSN_CLASS_I,   "d,s,>",   MATCH_SLLI, MASK_SLLI, match_opcode, INSN_ALIAS },
{"srli",        0, INSN_CLASS_C,   "Cs,Cw,C>",  MATCH_C_SRLI, MASK_C_SRLI, match_srxi_as_c_srxi, INSN_ALIAS },
{"srli",        0, INSN_CLASS_I,   "d,s,>",   MATCH_SRLI, MASK_SRLI, match_opcode, 0 },
{"srl",         0, INSN_CLASS_C,   "Cs,Cw,C>",  MATCH_C_SRLI, MASK_C_SRLI, match_srxi_as_c_srxi, INSN_ALIAS },
{"srl",         0, INSN_CLASS_I,   "d,s,t",   MATCH_SRL, MASK_SRL, match_opcode, 0 },
{"srl",         0, INSN_CLASS_I,   "d,s,>",   MATCH_SRLI, MASK_SRLI, match_opcode, INSN_ALIAS },
{"srai",        0, INSN_CLASS_C,   "Cs,Cw,C>",  MATCH_C_SRAI, MASK_C_SRAI, match_srxi_as_c_srxi, INSN_ALIAS },
{"srai",        0, INSN_CLASS_I,   "d,s,>",   MATCH_SRAI, MASK_SRAI, match_opcode, 0 },
{"sra",         0, INSN_CLASS_C,   "Cs,Cw,C>",  MATCH_C_SRAI, MASK_C_SRAI, match_srxi_as_c_srxi, INSN_ALIAS },
{"sra",         0, INSN_CLASS_I,   "d,s,t",   MATCH_SRA, MASK_SRA, match_opcode, 0 },
{"sra",         0, INSN_CLASS_I,   "d,s,>",   MATCH_SRAI, MASK_SRAI, match_opcode, INSN_ALIAS },
{"sub",         0, INSN_CLASS_C,   "Cs,Cw,Ct",  MATCH_C_SUB, MASK_C_SUB, match_opcode, INSN_ALIAS },
{"sub",         0, INSN_CLASS_I,   "d,s,t",  MATCH_SUB, MASK_SUB, match_opcode, 0 },
{"lb",          0, INSN_CLASS_I,   "d,o(s)",  MATCH_LB, MASK_LB, match_opcode, INSN_DREF|INSN_1_BYTE },
{"lb",          0, INSN_CLASS_I,   "d,A",  0, (int) M_LB, match_never, INSN_MACRO },
{"lbu",         0, INSN_CLASS_I,   "d,o(s)",  MATCH_LBU, MASK_LBU, match_opcode, INSN_DREF|INSN_1_BYTE },
{"lbu",         0, INSN_CLASS_I,   "d,A",  0, (int) M_LBU, match_never, INSN_MACRO },
{"lh",          0, INSN_CLASS_I,   "d,o(s)",  MATCH_LH, MASK_LH, match_opcode, INSN_DREF|INSN_2_BYTE },
{"lh",          0, INSN_CLASS_I,   "d,A",  0, (int) M_LH, match_never, INSN_MACRO },
{"lhu",         0, INSN_CLASS_I,   "d,o(s)",  MATCH_LHU, MASK_LHU, match_opcode, INSN_DREF|INSN_2_BYTE },
{"lhu",         0, INSN_CLASS_I,   "d,A",  0, (int) M_LHU, match_never, INSN_MACRO },
{"lw",          0, INSN_CLASS_C,   "d,Cm(Cc)",  MATCH_C_LWSP, MASK_C_LWSP, match_rd_nonzero, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"lw",          0, INSN_CLASS_C,   "Ct,Ck(Cs)",  MATCH_C_LW, MASK_C_LW, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"lw",          0, INSN_CLASS_I,   "d,o(s)",  MATCH_LW, MASK_LW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lw",          0, INSN_CLASS_I,   "d,A",  0, (int) M_LW, match_never, INSN_MACRO },
{"not",         0, INSN_CLASS_I,   "d,s",  MATCH_XORI | MASK_IMM, MASK_XORI | MASK_IMM, match_opcode, INSN_ALIAS },
{"ori",         0, INSN_CLASS_I,   "d,s,j",  MATCH_ORI, MASK_ORI, match_opcode, 0 },
{"or",          0, INSN_CLASS_C,   "Cs,Cw,Ct",  MATCH_C_OR, MASK_C_OR, match_opcode, INSN_ALIAS },
{"or",          0, INSN_CLASS_C,   "Cs,Ct,Cw",  MATCH_C_OR, MASK_C_OR, match_opcode, INSN_ALIAS },
{"or",          0, INSN_CLASS_I,   "d,s,t",  MATCH_OR, MASK_OR, match_opcode, 0 },
{"or",          0, INSN_CLASS_I,   "d,s,j",  MATCH_ORI, MASK_ORI, match_opcode, INSN_ALIAS },
{"auipc",       0, INSN_CLASS_I,   "d,u",  MATCH_AUIPC, MASK_AUIPC, match_opcode, 0 },
{"seqz",        0, INSN_CLASS_I,   "d,s",  MATCH_SLTIU | ENCODE_ITYPE_IMM (1), MASK_SLTIU | MASK_IMM, match_opcode, INSN_ALIAS },
{"snez",        0, INSN_CLASS_I,   "d,t",  MATCH_SLTU, MASK_SLTU | MASK_RS1, match_opcode, INSN_ALIAS },
{"sltz",        0, INSN_CLASS_I,   "d,s",  MATCH_SLT, MASK_SLT | MASK_RS2, match_opcode, INSN_ALIAS },
{"sgtz",        0, INSN_CLASS_I,   "d,t",  MATCH_SLT, MASK_SLT | MASK_RS1, match_opcode, INSN_ALIAS },
{"slti",        0, INSN_CLASS_I,   "d,s,j",  MATCH_SLTI, MASK_SLTI, match_opcode, 0 },
{"slt",         0, INSN_CLASS_I,   "d,s,t",  MATCH_SLT, MASK_SLT, match_opcode, 0 },
{"slt",         0, INSN_CLASS_I,   "d,s,j",  MATCH_SLTI, MASK_SLTI, match_opcode, INSN_ALIAS },
{"sltiu",       0, INSN_CLASS_I,   "d,s,j",  MATCH_SLTIU, MASK_SLTIU, match_opcode, 0 },
{"sltu",        0, INSN_CLASS_I,   "d,s,t",  MATCH_SLTU, MASK_SLTU, match_opcode, 0 },
{"sltu",        0, INSN_CLASS_I,   "d,s,j",  MATCH_SLTIU, MASK_SLTIU, match_opcode, INSN_ALIAS },
{"sgt",         0, INSN_CLASS_I,   "d,t,s",  MATCH_SLT, MASK_SLT, match_opcode, INSN_ALIAS },
{"sgtu",        0, INSN_CLASS_I,   "d,t,s",  MATCH_SLTU, MASK_SLTU, match_opcode, INSN_ALIAS },
{"sb",          0, INSN_CLASS_I,   "t,q(s)",  MATCH_SB, MASK_SB, match_opcode, INSN_DREF|INSN_1_BYTE },
{"sb",          0, INSN_CLASS_I,   "t,A,s",  0, (int) M_SB, match_never, INSN_MACRO },
{"sh",          0, INSN_CLASS_I,   "t,q(s)",  MATCH_SH, MASK_SH, match_opcode, INSN_DREF|INSN_2_BYTE },
{"sh",          0, INSN_CLASS_I,   "t,A,s",  0, (int) M_SH, match_never, INSN_MACRO },
{"sw",          0, INSN_CLASS_C,   "CV,CM(Cc)",  MATCH_C_SWSP, MASK_C_SWSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"sw",          0, INSN_CLASS_C,   "Ct,Ck(Cs)",  MATCH_C_SW, MASK_C_SW, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"sw",          0, INSN_CLASS_I,   "t,q(s)",  MATCH_SW, MASK_SW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sw",          0, INSN_CLASS_I,   "t,A,s",  0, (int) M_SW, match_never, INSN_MACRO },
{"pause",       0, INSN_CLASS_ZIHINTPAUSE,   "",  MATCH_PAUSE, MASK_PAUSE, match_opcode, 0 },
{"fence",       0, INSN_CLASS_I,   "",  MATCH_FENCE | MASK_PRED | MASK_SUCC, MASK_FENCE | MASK_RD | MASK_RS1 | MASK_IMM, match_opcode, INSN_ALIAS },
{"fence",       0, INSN_CLASS_I,   "P,Q",  MATCH_FENCE, MASK_FENCE | MASK_RD | MASK_RS1 | (MASK_IMM & ~MASK_PRED & ~MASK_SUCC), match_opcode, 0 },
{"fence.i",     0, INSN_CLASS_ZIFENCEI,   "",  MATCH_FENCE_I, MASK_FENCE | MASK_RD | MASK_RS1 | MASK_IMM, match_opcode, 0 },
{"fence.tso",   0, INSN_CLASS_I,   "",  MATCH_FENCE_TSO, MASK_FENCE_TSO | MASK_RD | MASK_RS1, match_opcode, INSN_ALIAS },
{"rdcycle",     0, INSN_CLASS_I,   "d",  MATCH_RDCYCLE, MASK_RDCYCLE, match_opcode, INSN_ALIAS },
{"rdinstret",   0, INSN_CLASS_I,   "d",  MATCH_RDINSTRET, MASK_RDINSTRET, match_opcode, INSN_ALIAS },
{"rdtime",      0, INSN_CLASS_I,   "d",  MATCH_RDTIME, MASK_RDTIME, match_opcode, INSN_ALIAS },
{"rdcycleh",   32, INSN_CLASS_I,   "d",  MATCH_RDCYCLEH, MASK_RDCYCLEH, match_opcode, INSN_ALIAS },
{"rdinstreth", 32, INSN_CLASS_I,   "d",  MATCH_RDINSTRETH, MASK_RDINSTRETH, match_opcode, INSN_ALIAS },
{"rdtimeh",    32, INSN_CLASS_I,   "d",  MATCH_RDTIMEH, MASK_RDTIMEH, match_opcode, INSN_ALIAS },
{"ecall",       0, INSN_CLASS_I,   "",    MATCH_SCALL, MASK_SCALL, match_opcode, 0 },
{"scall",       0, INSN_CLASS_I,   "",    MATCH_SCALL, MASK_SCALL, match_opcode, 0 },
{"xori",        0, INSN_CLASS_I,   "d,s,j",  MATCH_XORI, MASK_XORI, match_opcode, 0 },
{"xor",         0, INSN_CLASS_C,   "Cs,Cw,Ct",  MATCH_C_XOR, MASK_C_XOR, match_opcode, INSN_ALIAS },
{"xor",         0, INSN_CLASS_C,   "Cs,Ct,Cw",  MATCH_C_XOR, MASK_C_XOR, match_opcode, INSN_ALIAS },
{"xor",         0, INSN_CLASS_I,   "d,s,t",  MATCH_XOR, MASK_XOR, match_opcode, 0 },
{"xor",         0, INSN_CLASS_I,   "d,s,j",  MATCH_XORI, MASK_XORI, match_opcode, INSN_ALIAS },
{"lwu",        64, INSN_CLASS_I, "d,o(s)",  MATCH_LWU, MASK_LWU, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lwu",        64, INSN_CLASS_I, "d,A",  0, (int) M_LWU, match_never, INSN_MACRO },
{"ld",         64, INSN_CLASS_C, "d,Cn(Cc)",  MATCH_C_LDSP, MASK_C_LDSP, match_rd_nonzero, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"ld",         64, INSN_CLASS_C, "Ct,Cl(Cs)",  MATCH_C_LD, MASK_C_LD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"ld",         64, INSN_CLASS_I, "d,o(s)", MATCH_LD, MASK_LD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"ld",         64, INSN_CLASS_I, "d,A",  0, (int) M_LD, match_never, INSN_MACRO },
{"ld",         32, INSN_CLASS_XAMETHYST,   "d,Cm(Cc)",  MATCH_C_LDSP, MASK_C_LDSP, match_rd_nonzero, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"ld",         32, INSN_CLASS_XAMETHYST,   "Ct,Ck(Cs)",  MATCH_C_LD, MASK_C_LD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"ld",         32, INSN_CLASS_XAMETHYST,   "d,o(s)",  MATCH_LD, MASK_LD, match_opcode, INSN_DREF|INSN_4_BYTE },
{"ld",         32, INSN_CLASS_XAMETHYST,   "d,A",  0, (int) M_LD, match_never, INSN_MACRO },
{"sd",         64, INSN_CLASS_C, "CV,CN(Cc)",  MATCH_C_SDSP, MASK_C_SDSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"sd",         64, INSN_CLASS_C, "Ct,Cl(Cs)",  MATCH_C_SD, MASK_C_SD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"sd",         64, INSN_CLASS_I, "t,q(s)",  MATCH_SD, MASK_SD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"sd",         64, INSN_CLASS_I, "t,A,s",  0, (int) M_SD, match_never, INSN_MACRO },
{"sd",         32, INSN_CLASS_XAMETHYST,   "CV,CM(Cc)",  MATCH_C_SDSP, MASK_C_SDSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"sd",         32, INSN_CLASS_XAMETHYST,   "Ct,Ck(Cs)",  MATCH_C_SD, MASK_C_SD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"sd",         32, INSN_CLASS_XAMETHYST,   "t,q(s)",  MATCH_SD, MASK_SD, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sd",         32, INSN_CLASS_XAMETHYST,   "t,A,s",  0, (int) M_SD, match_never, INSN_MACRO },
{"zext.w",   64, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_PACK, MASK_PACK | MASK_RS2, match_opcode, 0 },
{"zext.w",     64, INSN_CLASS_I, "d,s",    0, (int) M_ZEXTW, match_never, INSN_MACRO },
{"sext.w",     64, INSN_CLASS_C, "d,CU",  MATCH_C_ADDIW, MASK_C_ADDIW | MASK_RVC_IMM, match_rd_nonzero, INSN_ALIAS },
{"sext.w",     64, INSN_CLASS_I, "d,s",  MATCH_ADDIW, MASK_ADDIW | MASK_IMM, match_opcode, INSN_ALIAS },
{"addiw",      64, INSN_CLASS_C, "d,CU,Co",  MATCH_C_ADDIW, MASK_C_ADDIW, match_rd_nonzero, INSN_ALIAS },
{"addiw",      64, INSN_CLASS_I, "d,s,j",  MATCH_ADDIW, MASK_ADDIW, match_opcode, 0 },
{"addw",       64, INSN_CLASS_C, "Cs,Cw,Ct",  MATCH_C_ADDW, MASK_C_ADDW, match_opcode, INSN_ALIAS },
{"addw",       64, INSN_CLASS_C, "Cs,Ct,Cw",  MATCH_C_ADDW, MASK_C_ADDW, match_opcode, INSN_ALIAS },
{"addw",       64, INSN_CLASS_C, "d,CU,Co",  MATCH_C_ADDIW, MASK_C_ADDIW, match_rd_nonzero, INSN_ALIAS },
{"addw",       64, INSN_CLASS_I, "d,s,t",  MATCH_ADDW, MASK_ADDW, match_opcode, 0 },
{"addw",       64, INSN_CLASS_I, "d,s,j",  MATCH_ADDIW, MASK_ADDIW, match_opcode, INSN_ALIAS },
{"negw",       64, INSN_CLASS_I, "d,t",  MATCH_SUBW, MASK_SUBW | MASK_RS1, match_opcode, INSN_ALIAS }, /* sub 0 */
{"slliw",      64, INSN_CLASS_I, "d,s,<",   MATCH_SLLIW, MASK_SLLIW, match_opcode, 0 },
{"sllw",       64, INSN_CLASS_I, "d,s,t",   MATCH_SLLW, MASK_SLLW, match_opcode, 0 },
{"sllw",       64, INSN_CLASS_I, "d,s,<",   MATCH_SLLIW, MASK_SLLIW, match_opcode, INSN_ALIAS },
{"srliw",      64, INSN_CLASS_I, "d,s,<",   MATCH_SRLIW, MASK_SRLIW, match_opcode, 0 },
{"srlw",       64, INSN_CLASS_I, "d,s,t",   MATCH_SRLW, MASK_SRLW, match_opcode, 0 },
{"srlw",       64, INSN_CLASS_I, "d,s,<",   MATCH_SRLIW, MASK_SRLIW, match_opcode, INSN_ALIAS },
{"sraiw",      64, INSN_CLASS_I, "d,s,<",   MATCH_SRAIW, MASK_SRAIW, match_opcode, 0 },
{"sraw",       64, INSN_CLASS_I, "d,s,t",   MATCH_SRAW, MASK_SRAW, match_opcode, 0 },
{"sraw",       64, INSN_CLASS_I, "d,s,<",   MATCH_SRAIW, MASK_SRAIW, match_opcode, INSN_ALIAS },
{"subw",       64, INSN_CLASS_C, "Cs,Cw,Ct",  MATCH_C_SUBW, MASK_C_SUBW, match_opcode, INSN_ALIAS },
{"subw",       64, INSN_CLASS_I, "d,s,t",  MATCH_SUBW, MASK_SUBW, match_opcode, 0 },

/* Atomic memory operation instruction subset */
{"lr.w",         0, INSN_CLASS_A,   "d,0(s)",    MATCH_LR_W, MASK_LR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sc.w",         0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_SC_W, MASK_SC_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoadd.w",     0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOADD_W, MASK_AMOADD_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoswap.w",    0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOSWAP_W, MASK_AMOSWAP_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoand.w",     0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOAND_W, MASK_AMOAND_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoor.w",      0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOOR_W, MASK_AMOOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoxor.w",     0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOXOR_W, MASK_AMOXOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomax.w",     0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAX_W, MASK_AMOMAX_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomaxu.w",    0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAXU_W, MASK_AMOMAXU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomin.w",     0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMIN_W, MASK_AMOMIN_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amominu.w",    0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMINU_W, MASK_AMOMINU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lr.w.aq",      0, INSN_CLASS_A,   "d,0(s)",    MATCH_LR_W | MASK_AQ, MASK_LR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sc.w.aq",      0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_SC_W | MASK_AQ, MASK_SC_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoadd.w.aq",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOADD_W | MASK_AQ, MASK_AMOADD_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoswap.w.aq", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOSWAP_W | MASK_AQ, MASK_AMOSWAP_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoand.w.aq",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOAND_W | MASK_AQ, MASK_AMOAND_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoor.w.aq",   0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOOR_W | MASK_AQ, MASK_AMOOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoxor.w.aq",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOXOR_W | MASK_AQ, MASK_AMOXOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomax.w.aq",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAX_W | MASK_AQ, MASK_AMOMAX_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomaxu.w.aq", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAXU_W | MASK_AQ, MASK_AMOMAXU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomin.w.aq",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMIN_W | MASK_AQ, MASK_AMOMIN_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amominu.w.aq", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMINU_W | MASK_AQ, MASK_AMOMINU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lr.w.rl",      0, INSN_CLASS_A,   "d,0(s)",    MATCH_LR_W | MASK_RL, MASK_LR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sc.w.rl",      0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_SC_W | MASK_RL, MASK_SC_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoadd.w.rl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOADD_W | MASK_RL, MASK_AMOADD_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoswap.w.rl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOSWAP_W | MASK_RL, MASK_AMOSWAP_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoand.w.rl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOAND_W | MASK_RL, MASK_AMOAND_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoor.w.rl",   0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOOR_W | MASK_RL, MASK_AMOOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoxor.w.rl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOXOR_W | MASK_RL, MASK_AMOXOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomax.w.rl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAX_W | MASK_RL, MASK_AMOMAX_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomaxu.w.rl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAXU_W | MASK_RL, MASK_AMOMAXU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomin.w.rl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMIN_W | MASK_RL, MASK_AMOMIN_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amominu.w.rl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMINU_W | MASK_RL, MASK_AMOMINU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lr.w.aqrl",    0, INSN_CLASS_A,   "d,0(s)",    MATCH_LR_W | MASK_AQRL, MASK_LR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"sc.w.aqrl",    0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_SC_W | MASK_AQRL, MASK_SC_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoadd.w.aqrl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOADD_W | MASK_AQRL, MASK_AMOADD_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoswap.w.aqrl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOSWAP_W | MASK_AQRL, MASK_AMOSWAP_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoand.w.aqrl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOAND_W | MASK_AQRL, MASK_AMOAND_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoor.w.aqrl",   0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOOR_W | MASK_AQRL, MASK_AMOOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amoxor.w.aqrl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOXOR_W | MASK_AQRL, MASK_AMOXOR_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomax.w.aqrl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAX_W | MASK_AQRL, MASK_AMOMAX_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomaxu.w.aqrl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMAXU_W | MASK_AQRL, MASK_AMOMAXU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amomin.w.aqrl",  0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMIN_W | MASK_AQRL, MASK_AMOMIN_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"amominu.w.aqrl", 0, INSN_CLASS_A,   "d,t,0(s)",  MATCH_AMOMINU_W | MASK_AQRL, MASK_AMOMINU_W | MASK_AQRL, match_opcode, INSN_DREF|INSN_4_BYTE },
{"lr.d",         64, INSN_CLASS_A , "d,0(s)",    MATCH_LR_D, MASK_LR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"sc.d",         64, INSN_CLASS_A , "d,t,0(s)",  MATCH_SC_D, MASK_SC_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoadd.d",     64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOADD_D, MASK_AMOADD_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoswap.d",    64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOSWAP_D, MASK_AMOSWAP_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoand.d",     64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOAND_D, MASK_AMOAND_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoor.d",      64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOOR_D, MASK_AMOOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoxor.d",     64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOXOR_D, MASK_AMOXOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomax.d",     64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAX_D, MASK_AMOMAX_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomaxu.d",    64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAXU_D, MASK_AMOMAXU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomin.d",     64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMIN_D, MASK_AMOMIN_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amominu.d",    64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMINU_D, MASK_AMOMINU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"lr.d.aq",      64, INSN_CLASS_A , "d,0(s)",    MATCH_LR_D | MASK_AQ, MASK_LR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"sc.d.aq",      64, INSN_CLASS_A , "d,t,0(s)",  MATCH_SC_D | MASK_AQ, MASK_SC_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoadd.d.aq",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOADD_D | MASK_AQ, MASK_AMOADD_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoswap.d.aq", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOSWAP_D | MASK_AQ, MASK_AMOSWAP_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoand.d.aq",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOAND_D | MASK_AQ, MASK_AMOAND_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoor.d.aq",   64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOOR_D | MASK_AQ, MASK_AMOOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoxor.d.aq",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOXOR_D | MASK_AQ, MASK_AMOXOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomax.d.aq",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAX_D | MASK_AQ, MASK_AMOMAX_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomaxu.d.aq", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAXU_D | MASK_AQ, MASK_AMOMAXU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomin.d.aq",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMIN_D | MASK_AQ, MASK_AMOMIN_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amominu.d.aq", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMINU_D | MASK_AQ, MASK_AMOMINU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"lr.d.rl",      64, INSN_CLASS_A , "d,0(s)",    MATCH_LR_D | MASK_RL, MASK_LR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"sc.d.rl",      64, INSN_CLASS_A , "d,t,0(s)",  MATCH_SC_D | MASK_RL, MASK_SC_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoadd.d.rl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOADD_D | MASK_RL, MASK_AMOADD_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoswap.d.rl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOSWAP_D | MASK_RL, MASK_AMOSWAP_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoand.d.rl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOAND_D | MASK_RL, MASK_AMOAND_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoor.d.rl",   64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOOR_D | MASK_RL, MASK_AMOOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoxor.d.rl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOXOR_D | MASK_RL, MASK_AMOXOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomax.d.rl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAX_D | MASK_RL, MASK_AMOMAX_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomaxu.d.rl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAXU_D | MASK_RL, MASK_AMOMAXU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomin.d.rl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMIN_D | MASK_RL, MASK_AMOMIN_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amominu.d.rl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMINU_D | MASK_RL, MASK_AMOMINU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"lr.d.aqrl",    64, INSN_CLASS_A , "d,0(s)",    MATCH_LR_D | MASK_AQRL, MASK_LR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"sc.d.aqrl",    64, INSN_CLASS_A , "d,t,0(s)",  MATCH_SC_D | MASK_AQRL, MASK_SC_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoadd.d.aqrl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOADD_D | MASK_AQRL, MASK_AMOADD_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoswap.d.aqrl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOSWAP_D | MASK_AQRL, MASK_AMOSWAP_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoand.d.aqrl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOAND_D | MASK_AQRL, MASK_AMOAND_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoor.d.aqrl",   64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOOR_D | MASK_AQRL, MASK_AMOOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amoxor.d.aqrl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOXOR_D | MASK_AQRL, MASK_AMOXOR_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomax.d.aqrl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAX_D | MASK_AQRL, MASK_AMOMAX_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomaxu.d.aqrl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMAXU_D | MASK_AQRL, MASK_AMOMAXU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amomin.d.aqrl",  64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMIN_D | MASK_AQRL, MASK_AMOMIN_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },
{"amominu.d.aqrl", 64, INSN_CLASS_A , "d,t,0(s)",  MATCH_AMOMINU_D | MASK_AQRL, MASK_AMOMINU_D | MASK_AQRL, match_opcode, INSN_DREF|INSN_8_BYTE },

/* Multiply/Divide instruction subset */
{"mul",       0, INSN_CLASS_M,   "d,s,t",  MATCH_MUL, MASK_MUL, match_opcode, 0 },
{"mulh",      0, INSN_CLASS_M,   "d,s,t",  MATCH_MULH, MASK_MULH, match_opcode, 0 },
{"mulhu",     0, INSN_CLASS_M,   "d,s,t",  MATCH_MULHU, MASK_MULHU, match_opcode, 0 },
{"mulhsu",    0, INSN_CLASS_M,   "d,s,t",  MATCH_MULHSU, MASK_MULHSU, match_opcode, 0 },
{"div",       0, INSN_CLASS_M,   "d,s,t",  MATCH_DIV, MASK_DIV, match_opcode, 0 },
{"divu",      0, INSN_CLASS_M,   "d,s,t",  MATCH_DIVU, MASK_DIVU, match_opcode, 0 },
{"rem",       0, INSN_CLASS_M,   "d,s,t",  MATCH_REM, MASK_REM, match_opcode, 0 },
{"remu",      0, INSN_CLASS_M,   "d,s,t",  MATCH_REMU, MASK_REMU, match_opcode, 0 },
{"mulw",     64, INSN_CLASS_M, "d,s,t",  MATCH_MULW, MASK_MULW, match_opcode, 0 },
{"divw",     64, INSN_CLASS_M, "d,s,t",  MATCH_DIVW, MASK_DIVW, match_opcode, 0 },
{"divuw",    64, INSN_CLASS_M, "d,s,t",  MATCH_DIVUW, MASK_DIVUW, match_opcode, 0 },
{"remw",     64, INSN_CLASS_M, "d,s,t",  MATCH_REMW, MASK_REMW, match_opcode, 0 },
{"remuw",    64, INSN_CLASS_M, "d,s,t",  MATCH_REMUW, MASK_REMUW, match_opcode, 0 },


/* Bitmanip instruction subset */
{"andn",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",  MATCH_ANDN, MASK_ANDN, match_opcode, 0 },
{"orn",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",  MATCH_ORN, MASK_ORN, match_opcode, 0 },
{"xnor",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",  MATCH_XNOR, MASK_XNOR, match_opcode, 0 },
{"sloi",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",   MATCH_SLOI, MASK_SLOI, match_opcode, 0 },
{"slo",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",   MATCH_SLO, MASK_SLO, match_opcode, 0 },
{"slo",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",   MATCH_SLOI, MASK_SLOI, match_opcode, INSN_ALIAS },
{"sroi",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",   MATCH_SROI, MASK_SROI, match_opcode, 0 },
{"sro",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",   MATCH_SRO, MASK_SRO, match_opcode, 0 },
{"sro",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",   MATCH_SROI, MASK_SROI, match_opcode, INSN_ALIAS },
{"rori",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",  MATCH_RORI, MASK_RORI, match_opcode, 0 },
{"rol",       0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_ROL, MASK_ROL, match_opcode, 0 },
{"ror",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",  MATCH_ROR, MASK_ROR, match_opcode, 0 },
{"ror",       0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,>",  MATCH_RORI, MASK_RORI, match_opcode, INSN_ALIAS },
{"bclri",     0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BCLRI, MASK_BCLRI, match_opcode, 0 },
{"bseti",     0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BSETI, MASK_BSETI, match_opcode, 0 },
{"binvi",     0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BINVI, MASK_BINVI, match_opcode, 0 },
{"bexti",     0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BEXTI, MASK_BEXTI, match_opcode, 0 },
{"gorci",     0, INSN_CLASS_B_OR_ZBP,   "d,s,>",  MATCH_GORCI, MASK_GORCI, match_opcode, 0 },
{"grevi",     0, INSN_CLASS_B_OR_ZBP,   "d,s,>",  MATCH_GREVI, MASK_GREVI, match_opcode, 0 },
{"bclr",      0, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BCLR, MASK_BCLR, match_opcode, 0 },
{"bclr",      0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BCLRI, MASK_BCLRI, match_opcode, INSN_ALIAS },
{"bset",      0, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BSET, MASK_BSET, match_opcode, 0 },
{"bset",      0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BSETI, MASK_BSETI, match_opcode, INSN_ALIAS },
{"binv",      0, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BINV, MASK_BINV, match_opcode, 0 },
{"binv",      0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BINVI, MASK_BINVI, match_opcode, INSN_ALIAS },
{"bext",      0, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BEXT, MASK_BEXT, match_opcode, 0 },
{"bext",      0, INSN_CLASS_B_OR_ZBS,   "d,s,>",  MATCH_BEXTI, MASK_BEXTI, match_opcode, INSN_ALIAS },
{"gorc",      0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_GORC, MASK_GORC, match_opcode, 0 },
{"gorc",      0, INSN_CLASS_B_OR_ZBP,   "d,s,>",  MATCH_GORCI, MASK_GORCI, match_opcode, INSN_ALIAS },
{"grev",      0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_GREV, MASK_GREV, match_opcode, 0 },
{"grev",      0, INSN_CLASS_B_OR_ZBP,   "d,s,>",  MATCH_GREVI, MASK_GREVI, match_opcode, INSN_ALIAS },
{"cmix",      0, INSN_CLASS_B_OR_ZBT,   "d,t,s,r",  MATCH_CMIX, MASK_CMIX, match_opcode, 0 },
{"cmov",      0, INSN_CLASS_B_OR_ZBT,   "d,t,s,r",  MATCH_CMOV, MASK_CMOV, match_opcode, 0 },
{"fsri",      0, INSN_CLASS_B_OR_ZBT,   "d,s,r,f",  MATCH_FSRI, MASK_FSRI, match_opcode, 0 },
{"fsl",       0, INSN_CLASS_B_OR_ZBT,   "d,s,r,t",  MATCH_FSL, MASK_FSL, match_opcode, 0 },
{"fsr",       0, INSN_CLASS_B_OR_ZBT,   "d,s,r,t",  MATCH_FSR, MASK_FSR, match_opcode, 0 },
{"fsr",       0, INSN_CLASS_B_OR_ZBT,   "d,s,r,f",  MATCH_FSRI, MASK_FSRI, match_opcode, INSN_ALIAS },
{"clz",       0, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CLZ, MASK_CLZ, match_opcode, 0 },
{"ctz",       0, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CTZ, MASK_CTZ, match_opcode, 0 },
{"cpop",      0, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CPOP, MASK_CPOP, match_opcode, 0 },
{"bmatflip", 64, INSN_CLASS_B_OR_ZBM,   "d,s",  MATCH_BMATFLIP, MASK_BMATFLIP, match_opcode, 0 },
{"crc32.b",   0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32_B, MASK_CRC32_B, match_opcode, 0 },
{"crc32.h",   0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32_H, MASK_CRC32_H, match_opcode, 0 },
{"crc32.w",   0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32_W, MASK_CRC32_W, match_opcode, 0 },
{"crc32.d",  64, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32_D, MASK_CRC32_D, match_opcode, 0 },
{"crc32c.b",  0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32C_B, MASK_CRC32C_B, match_opcode, 0 },
{"crc32c.h",  0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32C_H, MASK_CRC32C_H, match_opcode, 0 },
{"crc32c.w",  0, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32C_W, MASK_CRC32C_W, match_opcode, 0 },
{"crc32c.d", 64, INSN_CLASS_B_OR_ZBR,   "d,s",  MATCH_CRC32C_D, MASK_CRC32C_D, match_opcode, 0 },
{"sh1add",    0, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH1ADD, MASK_SH1ADD, match_opcode, 0 },
{"sh2add",    0, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH2ADD, MASK_SH2ADD, match_opcode, 0 },
{"sh3add",    0, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH3ADD, MASK_SH3ADD, match_opcode, 0 },
{"clmul",     0, INSN_CLASS_B_OR_ZBC,   "d,s,t",  MATCH_CLMUL, MASK_CLMUL, match_opcode, 0 },
{"clmulh",    0, INSN_CLASS_B_OR_ZBC,   "d,s,t",  MATCH_CLMULH, MASK_CLMULH, match_opcode, 0 },
{"clmulr",    0, INSN_CLASS_B_OR_ZBC,   "d,s,t",  MATCH_CLMULR, MASK_CLMULR, match_opcode, 0 },
{"min",       0, INSN_CLASS_B_OR_ZBB,   "d,s,t",  MATCH_MIN, MASK_MIN, match_opcode, 0 },
{"max",       0, INSN_CLASS_B_OR_ZBB,   "d,s,t",  MATCH_MAX, MASK_MAX, match_opcode, 0 },
{"minu",      0, INSN_CLASS_B_OR_ZBB,   "d,s,t",  MATCH_MINU, MASK_MINU, match_opcode, 0 },
{"maxu",      0, INSN_CLASS_B_OR_ZBB,   "d,s,t",  MATCH_MAXU, MASK_MAXU, match_opcode, 0 },
{"shfli",     0, INSN_CLASS_B_OR_ZBP,   "d,s,|",  MATCH_SHFLI, MASK_SHFLI, match_opcode, 0 },
{"unshfli",   0, INSN_CLASS_B_OR_ZBP,   "d,s,|",  MATCH_UNSHFLI, MASK_UNSHFLI, match_opcode, 0 },
{"shfl",      0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_SHFL, MASK_SHFL, match_opcode, 0 },
{"shfl",      0, INSN_CLASS_B_OR_ZBP,   "d,s,|",  MATCH_SHFLI, MASK_SHFLI, match_opcode, INSN_ALIAS },
{"unshfl",    0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_UNSHFL, MASK_UNSHFL, match_opcode, 0 },
{"unshfl",    0, INSN_CLASS_B_OR_ZBP,   "d,s,|",  MATCH_UNSHFLI, MASK_UNSHFLI, match_opcode, INSN_ALIAS },
{"bmdep",     0, INSN_CLASS_B_OR_ZBE,   "d,s,t",  MATCH_BMDEP, MASK_BMDEP, match_opcode, 0 },
{"bmext",     0, INSN_CLASS_B_OR_ZBE,   "d,s,t",  MATCH_BMEXT, MASK_BMEXT, match_opcode, 0 },
{"pack",      0, INSN_CLASS_B_OR_ZBF_OR_ZBP,   "d,s,t",  MATCH_PACK, MASK_PACK, match_opcode, 0 },
{"packu",     0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_PACKU, MASK_PACKU, match_opcode, 0 },
{"packh",     0, INSN_CLASS_B_OR_ZBF_OR_ZBP,   "d,s,t",  MATCH_PACKH, MASK_PACKH, match_opcode, 0 },
{"bmator",   64, INSN_CLASS_B_OR_ZBM,   "d,s,t",  MATCH_BMATOR, MASK_BMATOR, match_opcode, 0 },
{"bmatxor",  64, INSN_CLASS_B_OR_ZBM,   "d,s,t",  MATCH_BMATXOR, MASK_BMATXOR, match_opcode, 0 },
{"bfp",       0, INSN_CLASS_B_OR_ZBF,   "d,s,t",  MATCH_BFP, MASK_BFP, match_opcode, 0 },
{"slliu.w",  64, INSN_CLASS_B_OR_ZBA,   "d,s,>",  MATCH_SLLIU_W, MASK_SLLIU_W, match_opcode, 0 },
{"addu.w",   64, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_ADDU_W, MASK_ADDU_W, match_opcode, 0 },
{"roriw",    64, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,<",  MATCH_RORIW, MASK_RORIW, match_opcode, 0 },
{"rolw",     64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_ROLW, MASK_ROLW, match_opcode, 0 },
{"rorw",     64, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,t",  MATCH_RORW, MASK_RORW, match_opcode, 0 },
{"rorw",     64, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s,<",  MATCH_RORIW, MASK_RORIW, match_opcode, INSN_ALIAS },
{"bclriw",   64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BCLRIW, MASK_BCLRIW, match_opcode, 0 },
{"bsetiw",   64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BSETIW, MASK_BSETIW, match_opcode, 0 },
{"binviw",   64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BINVIW, MASK_BINVIW, match_opcode, 0 },
{"gorciw",   64, INSN_CLASS_B_OR_ZBP,   "d,s,<",  MATCH_GORCIW, MASK_GORCIW, match_opcode, 0 },
{"greviw",   64, INSN_CLASS_B_OR_ZBP,   "d,s,<",  MATCH_GREVIW, MASK_GREVIW, match_opcode, 0 },
{"bclrw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BCLRW, MASK_BCLRW, match_opcode, 0 },
{"bclrw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BCLRIW, MASK_BCLRIW, match_opcode, INSN_ALIAS },
{"bsetw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BSETW, MASK_BSETW, match_opcode, 0 },
{"bsetw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BSETIW, MASK_BSETIW, match_opcode, INSN_ALIAS },
{"binvw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BINVW, MASK_BINVW, match_opcode, 0 },
{"binvw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,<",  MATCH_BINVIW, MASK_BINVIW, match_opcode, INSN_ALIAS },
{"bextw",    64, INSN_CLASS_B_OR_ZBS,   "d,s,t",  MATCH_BEXTW, MASK_BEXTW, match_opcode, 0 },
{"gorcw",    64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_GORCW, MASK_GORCW, match_opcode, 0 },
{"gorcw",    64, INSN_CLASS_B_OR_ZBP,   "d,s,<",  MATCH_GORCIW, MASK_GORCIW, match_opcode, INSN_ALIAS },
{"grevw",    64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_GREVW, MASK_GREVW, match_opcode, 0 },
{"grevw",    64, INSN_CLASS_B_OR_ZBP,   "d,s,<",  MATCH_GREVIW, MASK_GREVIW, match_opcode, INSN_ALIAS },
{"fsriw",    64, INSN_CLASS_B_OR_ZBT,   "d,s,r,<",  MATCH_FSRIW, MASK_FSRIW, match_opcode, 0 },
{"fslw",     64, INSN_CLASS_B_OR_ZBT,   "d,s,r,t",  MATCH_FSLW, MASK_FSLW, match_opcode, 0 },
{"fsrw",     64, INSN_CLASS_B_OR_ZBT,   "d,s,r,t",  MATCH_FSRW, MASK_FSRW, match_opcode, 0 },
{"fsrw",     64, INSN_CLASS_B_OR_ZBT,   "d,s,r,<",  MATCH_FSRIW, MASK_FSRIW, match_opcode, INSN_ALIAS },
{"clzw",     64, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CLZW, MASK_CLZW, match_opcode, 0 },
{"ctzw",     64, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CTZW, MASK_CTZW, match_opcode, 0 },
{"cpopw",    64, INSN_CLASS_B_OR_ZBB,   "d,s",  MATCH_CPOPW, MASK_CPOPW, match_opcode, 0 },
{"sh1addu.w",64, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH1ADDU_W, MASK_SH1ADDU_W, match_opcode, 0 },
{"sh2addu.w",64, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH2ADDU_W, MASK_SH2ADDU_W, match_opcode, 0 },
{"sh3addu.w",64, INSN_CLASS_B_OR_ZBA,   "d,s,t",  MATCH_SH3ADDU_W, MASK_SH3ADDU_W, match_opcode, 0 },
{"shflw",    64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_SHFLW, MASK_SHFLW, match_opcode, 0 },
{"unshflw",  64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_UNSHFLW, MASK_UNSHFLW, match_opcode, 0 },
{"bmdepw",   64, INSN_CLASS_B_OR_ZBE,   "d,s,t",  MATCH_BMDEPW, MASK_BMDEPW, match_opcode, 0 },
{"bmextw",   64, INSN_CLASS_B_OR_ZBE,   "d,s,t",  MATCH_BMEXTW, MASK_BMEXTW, match_opcode, 0 },
{"packw",    64, INSN_CLASS_B_OR_ZBF_OR_ZBP,   "d,s,t",  MATCH_PACKW, MASK_PACKW, match_opcode, 0 },
{"packuw",   64, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_PACKUW, MASK_PACKUW, match_opcode, 0 },
{"bfpw",     64, INSN_CLASS_B_OR_ZBF,   "d,s,t",  MATCH_BFPW, MASK_BFPW, match_opcode, 0 },
{"xperm.n",   0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_XPERMN, MASK_XPERMN, match_opcode, 0 },
{"xperm.b",   0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_XPERMB, MASK_XPERMB, match_opcode, 0 },
{"xperm.h",   0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_XPERMH, MASK_XPERMH, match_opcode, 0 },
{"xperm.w",   0, INSN_CLASS_B_OR_ZBP,   "d,s,t",  MATCH_XPERMW, MASK_XPERMW, match_opcode, 0 },

/* Bitmanip pseudo-instructions */
{"rev.p",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev2.n",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev.n",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev4.b",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev2.b",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev.b",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev8.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev4.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev2.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev.h",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev16.w",  64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev8.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev4.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev2.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev.w",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev32",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev16",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev8",      0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev4",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev2",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"rev",       0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },

{"orc.p",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc2.n",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc.n",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc4.b",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc2.b",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc.b",     0, INSN_CLASS_B_OR_ZBB_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc8.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc4.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc2.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc.h",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc16.w",  64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc8.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc4.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc2.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc.w",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc32",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc16",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc8",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc4",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc2",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"orc",       0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },

{"zip.n",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip2.b",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip.b",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip4.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip2.h",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip.h",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip8.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip4.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip2.w",   64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip.w",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip16",    64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip8",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip4",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip2",      0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"zip",       0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },

{"unzip.n",   0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip2.b",  0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip.b",   0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip4.h",  0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip2.h",  0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip.h",   0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip8.w", 64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip4.w", 64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip2.w", 64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip.w",  64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip16",  64, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip8",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip4",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip2",    0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },
{"unzip",     0, INSN_CLASS_B_OR_ZBP,   "d,s",  0, (int) M_PERM, match_never, INSN_MACRO },

/* Cryptography instruction subset */
{"aes32esmi",   32, INSN_CLASS_K_OR_ZKNE, "s,t,b",  MATCH_AES32ESMI, MASK_AES32ESMI, match_opcode, 0 },
{"aes32esi",    32, INSN_CLASS_K_OR_ZKNE, "s,t,b",  MATCH_AES32ESI, MASK_AES32ESI, match_opcode, 0 },
{"aes32dsmi",   32, INSN_CLASS_K_OR_ZKND, "s,t,b",  MATCH_AES32DSMI, MASK_AES32DSMI, match_opcode, 0 },
{"aes32dsi",    32, INSN_CLASS_K_OR_ZKND, "s,t,b",  MATCH_AES32DSI, MASK_AES32DSI, match_opcode, 0 },
{"sha256sig0",   0, INSN_CLASS_K_OR_ZKNH, "d,s",  MATCH_SHA256SIG0, MASK_SHA256SIG0, match_opcode, 0 },
{"sha256sig1",   0, INSN_CLASS_K_OR_ZKNH, "d,s",  MATCH_SHA256SIG1, MASK_SHA256SIG1, match_opcode, 0 },
{"sha256sum0",   0, INSN_CLASS_K_OR_ZKNH, "d,s",  MATCH_SHA256SUM0, MASK_SHA256SUM0, match_opcode, 0 },
{"sha256sum1",   0, INSN_CLASS_K_OR_ZKNH, "d,s",  MATCH_SHA256SUM1, MASK_SHA256SUM1, match_opcode, 0 },
{"sha512sum0r", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SUM0R, MASK_SHA512SUM0R, match_opcode, 0 },
{"sha512sum1r", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SUM1R, MASK_SHA512SUM1R, match_opcode, 0 },
{"sha512sig0l", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SIG0L, MASK_SHA512SIG0L, match_opcode, 0 },
{"sha512sig0h", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SIG0H, MASK_SHA512SIG0H, match_opcode, 0 },
{"sha512sig1l", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SIG1L, MASK_SHA512SIG1L, match_opcode, 0 },
{"sha512sig1h", 32, INSN_CLASS_K_OR_ZKNH, "d,s,t",  MATCH_SHA512SIG1H, MASK_SHA512SIG1H, match_opcode, 0 },

/* Single-precision floating-point instruction subset */
{"frcsr",     0, INSN_CLASS_F,   "d",  MATCH_FRCSR, MASK_FRCSR, match_opcode, INSN_ALIAS },
{"frsr",      0, INSN_CLASS_F,   "d",  MATCH_FRCSR, MASK_FRCSR, match_opcode, INSN_ALIAS },
{"fscsr",     0, INSN_CLASS_F,   "s",  MATCH_FSCSR, MASK_FSCSR | MASK_RD, match_opcode, INSN_ALIAS },
{"fscsr",     0, INSN_CLASS_F,   "d,s",  MATCH_FSCSR, MASK_FSCSR, match_opcode, INSN_ALIAS },
{"fssr",      0, INSN_CLASS_F,   "s",  MATCH_FSCSR, MASK_FSCSR | MASK_RD, match_opcode, INSN_ALIAS },
{"fssr",      0, INSN_CLASS_F,   "d,s",  MATCH_FSCSR, MASK_FSCSR, match_opcode, INSN_ALIAS },
{"frrm",      0, INSN_CLASS_F,   "d",  MATCH_FRRM, MASK_FRRM, match_opcode, INSN_ALIAS },
{"fsrm",      0, INSN_CLASS_F,   "s",  MATCH_FSRM, MASK_FSRM | MASK_RD, match_opcode, INSN_ALIAS },
{"fsrm",      0, INSN_CLASS_F,   "d,s",  MATCH_FSRM, MASK_FSRM, match_opcode, INSN_ALIAS },
{"fsrmi",     0, INSN_CLASS_F,   "d,Z",  MATCH_FSRMI, MASK_FSRMI, match_opcode, INSN_ALIAS },
{"fsrmi",     0, INSN_CLASS_F,   "Z",  MATCH_FSRMI, MASK_FSRMI | MASK_RD, match_opcode, INSN_ALIAS },
{"frflags",   0, INSN_CLASS_F,   "d",  MATCH_FRFLAGS, MASK_FRFLAGS, match_opcode, INSN_ALIAS },
{"fsflags",   0, INSN_CLASS_F,   "s",  MATCH_FSFLAGS, MASK_FSFLAGS | MASK_RD, match_opcode, INSN_ALIAS },
{"fsflags",   0, INSN_CLASS_F,   "d,s",  MATCH_FSFLAGS, MASK_FSFLAGS, match_opcode, INSN_ALIAS },
{"fsflagsi",  0, INSN_CLASS_F,   "d,Z",  MATCH_FSFLAGSI, MASK_FSFLAGSI, match_opcode, INSN_ALIAS },
{"fsflagsi",  0, INSN_CLASS_F,   "Z",  MATCH_FSFLAGSI, MASK_FSFLAGSI | MASK_RD, match_opcode, INSN_ALIAS },
{"flw",      32, INSN_CLASS_F_AND_C, "D,Cm(Cc)",  MATCH_C_FLWSP, MASK_C_FLWSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"flw",      32, INSN_CLASS_F_AND_C, "CD,Ck(Cs)",  MATCH_C_FLW, MASK_C_FLW, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"flw",       0, INSN_CLASS_F,   "D,o(s)",  MATCH_FLW, MASK_FLW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"flw",       0, INSN_CLASS_F,   "D,A,s",  0, (int) M_FLW, match_never, INSN_MACRO },
{"fsw",      32, INSN_CLASS_F_AND_C, "CT,CM(Cc)",  MATCH_C_FSWSP, MASK_C_FSWSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"fsw",      32, INSN_CLASS_F_AND_C, "CD,Ck(Cs)",  MATCH_C_FSW, MASK_C_FSW, match_opcode, INSN_ALIAS|INSN_DREF|INSN_4_BYTE },
{"fsw",       0, INSN_CLASS_F,   "T,q(s)",  MATCH_FSW, MASK_FSW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"fsw",       0, INSN_CLASS_F,   "T,A,s",  0, (int) M_FSW, match_never, INSN_MACRO },

{"fmv.x.w",    0, INSN_CLASS_F,   "d,S",  MATCH_FMV_X_S, MASK_FMV_X_S, match_opcode, 0 },
{"fmv.w.x",    0, INSN_CLASS_F,   "D,s",  MATCH_FMV_S_X, MASK_FMV_S_X, match_opcode, 0 },

{"fmv.x.s",    0, INSN_CLASS_F,   "d,S",  MATCH_FMV_X_S, MASK_FMV_X_S, match_opcode, 0 },
{"fmv.s.x",    0, INSN_CLASS_F,   "D,s",  MATCH_FMV_S_X, MASK_FMV_S_X, match_opcode, 0 },

{"fmv.s",      0, INSN_CLASS_F,   "D,U",  MATCH_FSGNJ_S, MASK_FSGNJ_S, match_rs1_eq_rs2, INSN_ALIAS },
{"fneg.s",     0, INSN_CLASS_F,   "D,U",  MATCH_FSGNJN_S, MASK_FSGNJN_S, match_rs1_eq_rs2, INSN_ALIAS },
{"fabs.s",     0, INSN_CLASS_F,   "D,U",  MATCH_FSGNJX_S, MASK_FSGNJX_S, match_rs1_eq_rs2, INSN_ALIAS },
{"fsgnj.s",    0, INSN_CLASS_F,   "D,S,T",  MATCH_FSGNJ_S, MASK_FSGNJ_S, match_opcode, 0 },
{"fsgnjn.s",   0, INSN_CLASS_F,   "D,S,T",  MATCH_FSGNJN_S, MASK_FSGNJN_S, match_opcode, 0 },
{"fsgnjx.s",   0, INSN_CLASS_F,   "D,S,T",  MATCH_FSGNJX_S, MASK_FSGNJX_S, match_opcode, 0 },
{"fadd.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FADD_S | MASK_RM, MASK_FADD_S | MASK_RM, match_opcode, 0 },
{"fadd.s",     0, INSN_CLASS_F,   "D,S,T,m",  MATCH_FADD_S, MASK_FADD_S, match_opcode, 0 },
{"fsub.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FSUB_S | MASK_RM, MASK_FSUB_S | MASK_RM, match_opcode, 0 },
{"fsub.s",     0, INSN_CLASS_F,   "D,S,T,m",  MATCH_FSUB_S, MASK_FSUB_S, match_opcode, 0 },
{"fmul.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FMUL_S | MASK_RM, MASK_FMUL_S | MASK_RM, match_opcode, 0 },
{"fmul.s",     0, INSN_CLASS_F,   "D,S,T,m",  MATCH_FMUL_S, MASK_FMUL_S, match_opcode, 0 },
{"fdiv.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FDIV_S | MASK_RM, MASK_FDIV_S | MASK_RM, match_opcode, 0 },
{"fdiv.s",     0, INSN_CLASS_F,   "D,S,T,m",  MATCH_FDIV_S, MASK_FDIV_S, match_opcode, 0 },
{"fsqrt.s",    0, INSN_CLASS_F,   "D,S",  MATCH_FSQRT_S | MASK_RM, MASK_FSQRT_S | MASK_RM, match_opcode, 0 },
{"fsqrt.s",    0, INSN_CLASS_F,   "D,S,m",  MATCH_FSQRT_S, MASK_FSQRT_S, match_opcode, 0 },
{"fmin.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FMIN_S, MASK_FMIN_S, match_opcode, 0 },
{"fmax.s",     0, INSN_CLASS_F,   "D,S,T",  MATCH_FMAX_S, MASK_FMAX_S, match_opcode, 0 },
{"fmadd.s",    0, INSN_CLASS_F,   "D,S,T,R",  MATCH_FMADD_S | MASK_RM, MASK_FMADD_S | MASK_RM, match_opcode, 0 },
{"fmadd.s",    0, INSN_CLASS_F,   "D,S,T,R,m",  MATCH_FMADD_S, MASK_FMADD_S, match_opcode, 0 },
{"fnmadd.s",   0, INSN_CLASS_F,   "D,S,T,R",  MATCH_FNMADD_S | MASK_RM, MASK_FNMADD_S | MASK_RM, match_opcode, 0 },
{"fnmadd.s",   0, INSN_CLASS_F,   "D,S,T,R,m",  MATCH_FNMADD_S, MASK_FNMADD_S, match_opcode, 0 },
{"fmsub.s",    0, INSN_CLASS_F,   "D,S,T,R",  MATCH_FMSUB_S | MASK_RM, MASK_FMSUB_S | MASK_RM, match_opcode, 0 },
{"fmsub.s",    0, INSN_CLASS_F,   "D,S,T,R,m",  MATCH_FMSUB_S, MASK_FMSUB_S, match_opcode, 0 },
{"fnmsub.s",   0, INSN_CLASS_F,   "D,S,T,R",  MATCH_FNMSUB_S | MASK_RM, MASK_FNMSUB_S | MASK_RM, match_opcode, 0 },
{"fnmsub.s",   0, INSN_CLASS_F,   "D,S,T,R,m",  MATCH_FNMSUB_S, MASK_FNMSUB_S, match_opcode, 0 },
{"fcvt.w.s",   0, INSN_CLASS_F,   "d,S",  MATCH_FCVT_W_S | MASK_RM, MASK_FCVT_W_S | MASK_RM, match_opcode, 0 },
{"fcvt.w.s",   0, INSN_CLASS_F,   "d,S,m",  MATCH_FCVT_W_S, MASK_FCVT_W_S, match_opcode, 0 },
{"fcvt.wu.s",  0, INSN_CLASS_F,   "d,S",  MATCH_FCVT_WU_S | MASK_RM, MASK_FCVT_WU_S | MASK_RM, match_opcode, 0 },
{"fcvt.wu.s",  0, INSN_CLASS_F,   "d,S,m",  MATCH_FCVT_WU_S, MASK_FCVT_WU_S, match_opcode, 0 },
{"fcvt.s.w",   0, INSN_CLASS_F,   "D,s",  MATCH_FCVT_S_W | MASK_RM, MASK_FCVT_S_W | MASK_RM, match_opcode, 0 },
{"fcvt.s.w",   0, INSN_CLASS_F,   "D,s,m",  MATCH_FCVT_S_W, MASK_FCVT_S_W, match_opcode, 0 },
{"fcvt.s.wu",  0, INSN_CLASS_F,   "D,s",  MATCH_FCVT_S_WU | MASK_RM, MASK_FCVT_S_W | MASK_RM, match_opcode, 0 },
{"fcvt.s.wu",  0, INSN_CLASS_F,   "D,s,m",  MATCH_FCVT_S_WU, MASK_FCVT_S_WU, match_opcode, 0 },
{"fclass.s",   0, INSN_CLASS_F,   "d,S",  MATCH_FCLASS_S, MASK_FCLASS_S, match_opcode, 0 },
{"feq.s",      0, INSN_CLASS_F,   "d,S,T",    MATCH_FEQ_S, MASK_FEQ_S, match_opcode, 0 },
{"flt.s",      0, INSN_CLASS_F,   "d,S,T",    MATCH_FLT_S, MASK_FLT_S, match_opcode, 0 },
{"fle.s",      0, INSN_CLASS_F,   "d,S,T",    MATCH_FLE_S, MASK_FLE_S, match_opcode, 0 },
{"fgt.s",      0, INSN_CLASS_F,   "d,T,S",    MATCH_FLT_S, MASK_FLT_S, match_opcode, 0 },
{"fge.s",      0, INSN_CLASS_F,   "d,T,S",    MATCH_FLE_S, MASK_FLE_S, match_opcode, 0 },
{"fcvt.l.s",  64, INSN_CLASS_F, "d,S",  MATCH_FCVT_L_S | MASK_RM, MASK_FCVT_L_S | MASK_RM, match_opcode, 0 },
{"fcvt.l.s",  64, INSN_CLASS_F, "d,S,m",  MATCH_FCVT_L_S, MASK_FCVT_L_S, match_opcode, 0 },
{"fcvt.lu.s", 64, INSN_CLASS_F, "d,S",  MATCH_FCVT_LU_S | MASK_RM, MASK_FCVT_LU_S | MASK_RM, match_opcode, 0 },
{"fcvt.lu.s", 64, INSN_CLASS_F, "d,S,m",  MATCH_FCVT_LU_S, MASK_FCVT_LU_S, match_opcode, 0 },
{"fcvt.s.l",  64, INSN_CLASS_F, "D,s",  MATCH_FCVT_S_L | MASK_RM, MASK_FCVT_S_L | MASK_RM, match_opcode, 0 },
{"fcvt.s.l",  64, INSN_CLASS_F, "D,s,m",  MATCH_FCVT_S_L, MASK_FCVT_S_L, match_opcode, 0 },
{"fcvt.s.lu", 64, INSN_CLASS_F, "D,s",  MATCH_FCVT_S_LU | MASK_RM, MASK_FCVT_S_L | MASK_RM, match_opcode, 0 },
{"fcvt.s.lu", 64, INSN_CLASS_F, "D,s,m",  MATCH_FCVT_S_LU, MASK_FCVT_S_LU, match_opcode, 0 },

/* Double-precision floating-point instruction subset */
{"fld",        0, INSN_CLASS_D_AND_C,   "D,Cn(Cc)",  MATCH_C_FLDSP, MASK_C_FLDSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"fld",        0, INSN_CLASS_D_AND_C,   "CD,Cl(Cs)",  MATCH_C_FLD, MASK_C_FLD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"fld",        0, INSN_CLASS_D,   "D,o(s)",  MATCH_FLD, MASK_FLD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"fld",        0, INSN_CLASS_D,   "D,A,s",  0, (int) M_FLD, match_never, INSN_MACRO },
{"fsd",        0, INSN_CLASS_D_AND_C,   "CT,CN(Cc)",  MATCH_C_FSDSP, MASK_C_FSDSP, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"fsd",        0, INSN_CLASS_D_AND_C,   "CD,Cl(Cs)",  MATCH_C_FSD, MASK_C_FSD, match_opcode, INSN_ALIAS|INSN_DREF|INSN_8_BYTE },
{"fsd",        0, INSN_CLASS_D,   "T,q(s)",  MATCH_FSD, MASK_FSD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"fsd",        0, INSN_CLASS_D,   "T,A,s",  0, (int) M_FSD, match_never, INSN_MACRO },
{"fmv.d",      0, INSN_CLASS_D,   "D,U",  MATCH_FSGNJ_D, MASK_FSGNJ_D, match_rs1_eq_rs2, INSN_ALIAS },
{"fneg.d",     0, INSN_CLASS_D,   "D,U",  MATCH_FSGNJN_D, MASK_FSGNJN_D, match_rs1_eq_rs2, INSN_ALIAS },
{"fabs.d",     0, INSN_CLASS_D,   "D,U",  MATCH_FSGNJX_D, MASK_FSGNJX_D, match_rs1_eq_rs2, INSN_ALIAS },
{"fsgnj.d",    0, INSN_CLASS_D,   "D,S,T",  MATCH_FSGNJ_D, MASK_FSGNJ_D, match_opcode, 0 },
{"fsgnjn.d",   0, INSN_CLASS_D,   "D,S,T",  MATCH_FSGNJN_D, MASK_FSGNJN_D, match_opcode, 0 },
{"fsgnjx.d",   0, INSN_CLASS_D,   "D,S,T",  MATCH_FSGNJX_D, MASK_FSGNJX_D, match_opcode, 0 },
{"fadd.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FADD_D | MASK_RM, MASK_FADD_D | MASK_RM, match_opcode, 0 },
{"fadd.d",     0, INSN_CLASS_D,   "D,S,T,m",  MATCH_FADD_D, MASK_FADD_D, match_opcode, 0 },
{"fsub.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FSUB_D | MASK_RM, MASK_FSUB_D | MASK_RM, match_opcode, 0 },
{"fsub.d",     0, INSN_CLASS_D,   "D,S,T,m",  MATCH_FSUB_D, MASK_FSUB_D, match_opcode, 0 },
{"fmul.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FMUL_D | MASK_RM, MASK_FMUL_D | MASK_RM, match_opcode, 0 },
{"fmul.d",     0, INSN_CLASS_D,   "D,S,T,m",  MATCH_FMUL_D, MASK_FMUL_D, match_opcode, 0 },
{"fdiv.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FDIV_D | MASK_RM, MASK_FDIV_D | MASK_RM, match_opcode, 0 },
{"fdiv.d",     0, INSN_CLASS_D,   "D,S,T,m",  MATCH_FDIV_D, MASK_FDIV_D, match_opcode, 0 },
{"fsqrt.d",    0, INSN_CLASS_D,   "D,S",  MATCH_FSQRT_D | MASK_RM, MASK_FSQRT_D | MASK_RM, match_opcode, 0 },
{"fsqrt.d",    0, INSN_CLASS_D,   "D,S,m",  MATCH_FSQRT_D, MASK_FSQRT_D, match_opcode, 0 },
{"fmin.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FMIN_D, MASK_FMIN_D, match_opcode, 0 },
{"fmax.d",     0, INSN_CLASS_D,   "D,S,T",  MATCH_FMAX_D, MASK_FMAX_D, match_opcode, 0 },
{"fmadd.d",    0, INSN_CLASS_D,   "D,S,T,R",  MATCH_FMADD_D | MASK_RM, MASK_FMADD_D | MASK_RM, match_opcode, 0 },
{"fmadd.d",    0, INSN_CLASS_D,   "D,S,T,R,m",  MATCH_FMADD_D, MASK_FMADD_D, match_opcode, 0 },
{"fnmadd.d",   0, INSN_CLASS_D,   "D,S,T,R",  MATCH_FNMADD_D | MASK_RM, MASK_FNMADD_D | MASK_RM, match_opcode, 0 },
{"fnmadd.d",   0, INSN_CLASS_D,   "D,S,T,R,m",  MATCH_FNMADD_D, MASK_FNMADD_D, match_opcode, 0 },
{"fmsub.d",    0, INSN_CLASS_D,   "D,S,T,R",  MATCH_FMSUB_D | MASK_RM, MASK_FMSUB_D | MASK_RM, match_opcode, 0 },
{"fmsub.d",    0, INSN_CLASS_D,   "D,S,T,R,m",  MATCH_FMSUB_D, MASK_FMSUB_D, match_opcode, 0 },
{"fnmsub.d",   0, INSN_CLASS_D,   "D,S,T,R",  MATCH_FNMSUB_D | MASK_RM, MASK_FNMSUB_D | MASK_RM, match_opcode, 0 },
{"fnmsub.d",   0, INSN_CLASS_D,   "D,S,T,R,m",  MATCH_FNMSUB_D, MASK_FNMSUB_D, match_opcode, 0 },
{"fcvt.w.d",   0, INSN_CLASS_D,   "d,S",  MATCH_FCVT_W_D | MASK_RM, MASK_FCVT_W_D | MASK_RM, match_opcode, 0 },
{"fcvt.w.d",   0, INSN_CLASS_D,   "d,S,m",  MATCH_FCVT_W_D, MASK_FCVT_W_D, match_opcode, 0 },
{"fcvt.wu.d",  0, INSN_CLASS_D,   "d,S",  MATCH_FCVT_WU_D | MASK_RM, MASK_FCVT_WU_D | MASK_RM, match_opcode, 0 },
{"fcvt.wu.d",  0, INSN_CLASS_D,   "d,S,m",  MATCH_FCVT_WU_D, MASK_FCVT_WU_D, match_opcode, 0 },
{"fcvt.d.w",   0, INSN_CLASS_D,   "D,s",  MATCH_FCVT_D_W, MASK_FCVT_D_W | MASK_RM, match_opcode, 0 },
{"fcvt.d.wu",  0, INSN_CLASS_D,   "D,s",  MATCH_FCVT_D_WU, MASK_FCVT_D_WU | MASK_RM, match_opcode, 0 },
{"fcvt.d.s",   0, INSN_CLASS_D,   "D,S",  MATCH_FCVT_D_S, MASK_FCVT_D_S | MASK_RM, match_opcode, 0 },
{"fcvt.s.d",   0, INSN_CLASS_D,   "D,S",  MATCH_FCVT_S_D | MASK_RM, MASK_FCVT_S_D | MASK_RM, match_opcode, 0 },
{"fcvt.s.d",   0, INSN_CLASS_D,   "D,S,m",  MATCH_FCVT_S_D, MASK_FCVT_S_D, match_opcode, 0 },
{"fclass.d",   0, INSN_CLASS_D,   "d,S",  MATCH_FCLASS_D, MASK_FCLASS_D, match_opcode, 0 },
{"feq.d",      0, INSN_CLASS_D,   "d,S,T",    MATCH_FEQ_D, MASK_FEQ_D, match_opcode, 0 },
{"flt.d",      0, INSN_CLASS_D,   "d,S,T",    MATCH_FLT_D, MASK_FLT_D, match_opcode, 0 },
{"fle.d",      0, INSN_CLASS_D,   "d,S,T",    MATCH_FLE_D, MASK_FLE_D, match_opcode, 0 },
{"fgt.d",      0, INSN_CLASS_D,   "d,T,S",    MATCH_FLT_D, MASK_FLT_D, match_opcode, 0 },
{"fge.d",      0, INSN_CLASS_D,   "d,T,S",    MATCH_FLE_D, MASK_FLE_D, match_opcode, 0 },
{"fmv.x.d",   64, INSN_CLASS_D, "d,S",  MATCH_FMV_X_D, MASK_FMV_X_D, match_opcode, 0 },
{"fmv.d.x",   64, INSN_CLASS_D, "D,s",  MATCH_FMV_D_X, MASK_FMV_D_X, match_opcode, 0 },
{"fcvt.l.d",  64, INSN_CLASS_D, "d,S",  MATCH_FCVT_L_D | MASK_RM, MASK_FCVT_L_D | MASK_RM, match_opcode, 0 },
{"fcvt.l.d",  64, INSN_CLASS_D, "d,S,m",  MATCH_FCVT_L_D, MASK_FCVT_L_D, match_opcode, 0 },
{"fcvt.lu.d", 64, INSN_CLASS_D, "d,S",  MATCH_FCVT_LU_D | MASK_RM, MASK_FCVT_LU_D | MASK_RM, match_opcode, 0 },
{"fcvt.lu.d", 64, INSN_CLASS_D, "d,S,m",  MATCH_FCVT_LU_D, MASK_FCVT_LU_D, match_opcode, 0 },
{"fcvt.d.l",  64, INSN_CLASS_D, "D,s",  MATCH_FCVT_D_L | MASK_RM, MASK_FCVT_D_L | MASK_RM, match_opcode, 0 },
{"fcvt.d.l",  64, INSN_CLASS_D, "D,s,m",  MATCH_FCVT_D_L, MASK_FCVT_D_L, match_opcode, 0 },
{"fcvt.d.lu", 64, INSN_CLASS_D, "D,s",  MATCH_FCVT_D_LU | MASK_RM, MASK_FCVT_D_L | MASK_RM, match_opcode, 0 },
{"fcvt.d.lu", 64, INSN_CLASS_D, "D,s,m",  MATCH_FCVT_D_LU, MASK_FCVT_D_LU, match_opcode, 0 },

/* Quad-precision floating-point instruction subset */
{"flq",        0, INSN_CLASS_Q,   "D,o(s)",  MATCH_FLQ, MASK_FLQ, match_opcode, INSN_DREF|INSN_16_BYTE },
{"flq",        0, INSN_CLASS_Q,   "D,A,s",  0, (int) M_FLQ, match_never, INSN_MACRO },
{"fsq",        0, INSN_CLASS_Q,   "T,q(s)",  MATCH_FSQ, MASK_FSQ, match_opcode, INSN_DREF|INSN_16_BYTE },
{"fsq",        0, INSN_CLASS_Q,   "T,A,s",  0, (int) M_FSQ, match_never, INSN_MACRO },
{"fmv.q",      0, INSN_CLASS_Q,   "D,U",  MATCH_FSGNJ_Q, MASK_FSGNJ_Q, match_rs1_eq_rs2, INSN_ALIAS },
{"fneg.q",     0, INSN_CLASS_Q,   "D,U",  MATCH_FSGNJN_Q, MASK_FSGNJN_Q, match_rs1_eq_rs2, INSN_ALIAS },
{"fabs.q",     0, INSN_CLASS_Q,   "D,U",  MATCH_FSGNJX_Q, MASK_FSGNJX_Q, match_rs1_eq_rs2, INSN_ALIAS },
{"fsgnj.q",    0, INSN_CLASS_Q,   "D,S,T",  MATCH_FSGNJ_Q, MASK_FSGNJ_Q, match_opcode, 0 },
{"fsgnjn.q",   0, INSN_CLASS_Q,   "D,S,T",  MATCH_FSGNJN_Q, MASK_FSGNJN_Q, match_opcode, 0 },
{"fsgnjx.q",   0, INSN_CLASS_Q,   "D,S,T",  MATCH_FSGNJX_Q, MASK_FSGNJX_Q, match_opcode, 0 },
{"fadd.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FADD_Q | MASK_RM, MASK_FADD_Q | MASK_RM, match_opcode, 0 },
{"fadd.q",     0, INSN_CLASS_Q,   "D,S,T,m",  MATCH_FADD_Q, MASK_FADD_Q, match_opcode, 0 },
{"fsub.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FSUB_Q | MASK_RM, MASK_FSUB_Q | MASK_RM, match_opcode, 0 },
{"fsub.q",     0, INSN_CLASS_Q,   "D,S,T,m",  MATCH_FSUB_Q, MASK_FSUB_Q, match_opcode, 0 },
{"fmul.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FMUL_Q | MASK_RM, MASK_FMUL_Q | MASK_RM, match_opcode, 0 },
{"fmul.q",     0, INSN_CLASS_Q,   "D,S,T,m",  MATCH_FMUL_Q, MASK_FMUL_Q, match_opcode, 0 },
{"fdiv.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FDIV_Q | MASK_RM, MASK_FDIV_Q | MASK_RM, match_opcode, 0 },
{"fdiv.q",     0, INSN_CLASS_Q,   "D,S,T,m",  MATCH_FDIV_Q, MASK_FDIV_Q, match_opcode, 0 },
{"fsqrt.q",    0, INSN_CLASS_Q,   "D,S",  MATCH_FSQRT_Q | MASK_RM, MASK_FSQRT_Q | MASK_RM, match_opcode, 0 },
{"fsqrt.q",    0, INSN_CLASS_Q,   "D,S,m",  MATCH_FSQRT_Q, MASK_FSQRT_Q, match_opcode, 0 },
{"fmin.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FMIN_Q, MASK_FMIN_Q, match_opcode, 0 },
{"fmax.q",     0, INSN_CLASS_Q,   "D,S,T",  MATCH_FMAX_Q, MASK_FMAX_Q, match_opcode, 0 },
{"fmadd.q",    0, INSN_CLASS_Q,   "D,S,T,R",  MATCH_FMADD_Q | MASK_RM, MASK_FMADD_Q | MASK_RM, match_opcode, 0 },
{"fmadd.q",    0, INSN_CLASS_Q,   "D,S,T,R,m",  MATCH_FMADD_Q, MASK_FMADD_Q, match_opcode, 0 },
{"fnmadd.q",   0, INSN_CLASS_Q,   "D,S,T,R",  MATCH_FNMADD_Q | MASK_RM, MASK_FNMADD_Q | MASK_RM, match_opcode, 0 },
{"fnmadd.q",   0, INSN_CLASS_Q,   "D,S,T,R,m",  MATCH_FNMADD_Q, MASK_FNMADD_Q, match_opcode, 0 },
{"fmsub.q",    0, INSN_CLASS_Q,   "D,S,T,R",  MATCH_FMSUB_Q | MASK_RM, MASK_FMSUB_Q | MASK_RM, match_opcode, 0 },
{"fmsub.q",    0, INSN_CLASS_Q,   "D,S,T,R,m",  MATCH_FMSUB_Q, MASK_FMSUB_Q, match_opcode, 0 },
{"fnmsub.q",   0, INSN_CLASS_Q,   "D,S,T,R",  MATCH_FNMSUB_Q | MASK_RM, MASK_FNMSUB_Q | MASK_RM, match_opcode, 0 },
{"fnmsub.q",   0, INSN_CLASS_Q,   "D,S,T,R,m",  MATCH_FNMSUB_Q, MASK_FNMSUB_Q, match_opcode, 0 },
{"fcvt.w.q",   0, INSN_CLASS_Q,   "d,S",  MATCH_FCVT_W_Q | MASK_RM, MASK_FCVT_W_Q | MASK_RM, match_opcode, 0 },
{"fcvt.w.q",   0, INSN_CLASS_Q,   "d,S,m",  MATCH_FCVT_W_Q, MASK_FCVT_W_Q, match_opcode, 0 },
{"fcvt.wu.q",  0, INSN_CLASS_Q,   "d,S",  MATCH_FCVT_WU_Q | MASK_RM, MASK_FCVT_WU_Q | MASK_RM, match_opcode, 0 },
{"fcvt.wu.q",  0, INSN_CLASS_Q,   "d,S,m",  MATCH_FCVT_WU_Q, MASK_FCVT_WU_Q, match_opcode, 0 },
{"fcvt.q.w",   0, INSN_CLASS_Q,   "D,s",  MATCH_FCVT_Q_W, MASK_FCVT_Q_W | MASK_RM, match_opcode, 0 },
{"fcvt.q.wu",  0, INSN_CLASS_Q,   "D,s",  MATCH_FCVT_Q_WU, MASK_FCVT_Q_WU | MASK_RM, match_opcode, 0 },
{"fcvt.q.s",   0, INSN_CLASS_Q,   "D,S",  MATCH_FCVT_Q_S, MASK_FCVT_Q_S | MASK_RM, match_opcode, 0 },
{"fcvt.q.d",   0, INSN_CLASS_Q,   "D,S",  MATCH_FCVT_Q_D, MASK_FCVT_Q_D | MASK_RM, match_opcode, 0 },
{"fcvt.s.q",   0, INSN_CLASS_Q,   "D,S",  MATCH_FCVT_S_Q | MASK_RM, MASK_FCVT_S_Q | MASK_RM, match_opcode, 0 },
{"fcvt.s.q",   0, INSN_CLASS_Q,   "D,S,m",  MATCH_FCVT_S_Q, MASK_FCVT_S_Q, match_opcode, 0 },
{"fcvt.d.q",   0, INSN_CLASS_Q,   "D,S",  MATCH_FCVT_D_Q | MASK_RM, MASK_FCVT_D_Q | MASK_RM, match_opcode, 0 },
{"fcvt.d.q",   0, INSN_CLASS_Q,   "D,S,m",  MATCH_FCVT_D_Q, MASK_FCVT_D_Q, match_opcode, 0 },
{"fclass.q",   0, INSN_CLASS_Q,   "d,S",  MATCH_FCLASS_Q, MASK_FCLASS_Q, match_opcode, 0 },
{"feq.q",      0, INSN_CLASS_Q,   "d,S,T",    MATCH_FEQ_Q, MASK_FEQ_Q, match_opcode, 0 },
{"flt.q",      0, INSN_CLASS_Q,   "d,S,T",    MATCH_FLT_Q, MASK_FLT_Q, match_opcode, 0 },
{"fle.q",      0, INSN_CLASS_Q,   "d,S,T",    MATCH_FLE_Q, MASK_FLE_Q, match_opcode, 0 },
{"fgt.q",      0, INSN_CLASS_Q,   "d,T,S",    MATCH_FLT_Q, MASK_FLT_Q, match_opcode, 0 },
{"fge.q",      0, INSN_CLASS_Q,   "d,T,S",    MATCH_FLE_Q, MASK_FLE_Q, match_opcode, 0 },
{"fmv.x.q",   64, INSN_CLASS_Q, "d,S",  MATCH_FMV_X_Q, MASK_FMV_X_Q, match_opcode, 0 },
{"fmv.q.x",   64, INSN_CLASS_Q, "D,s",  MATCH_FMV_Q_X, MASK_FMV_Q_X, match_opcode, 0 },
{"fcvt.l.q",  64, INSN_CLASS_Q, "d,S",  MATCH_FCVT_L_Q | MASK_RM, MASK_FCVT_L_Q | MASK_RM, match_opcode, 0 },
{"fcvt.l.q",  64, INSN_CLASS_Q, "d,S,m",  MATCH_FCVT_L_Q, MASK_FCVT_L_Q, match_opcode, 0 },
{"fcvt.lu.q", 64, INSN_CLASS_Q, "d,S",  MATCH_FCVT_LU_Q | MASK_RM, MASK_FCVT_LU_Q | MASK_RM, match_opcode, 0 },
{"fcvt.lu.q", 64, INSN_CLASS_Q, "d,S,m",  MATCH_FCVT_LU_Q, MASK_FCVT_LU_Q, match_opcode, 0 },
{"fcvt.q.l",  64, INSN_CLASS_Q, "D,s",  MATCH_FCVT_Q_L | MASK_RM, MASK_FCVT_Q_L | MASK_RM, match_opcode, 0 },
{"fcvt.q.l",  64, INSN_CLASS_Q, "D,s,m",  MATCH_FCVT_Q_L, MASK_FCVT_Q_L, match_opcode, 0 },
{"fcvt.q.lu", 64, INSN_CLASS_Q, "D,s",  MATCH_FCVT_Q_LU | MASK_RM, MASK_FCVT_Q_L | MASK_RM, match_opcode, 0 },
{"fcvt.q.lu", 64, INSN_CLASS_Q, "D,s,m",  MATCH_FCVT_Q_LU, MASK_FCVT_Q_LU, match_opcode, 0 },

/* Compressed instructions.  */
{"c.unimp",    0, INSN_CLASS_C,   "",  0, 0xffffU,  match_opcode, 0 },
{"c.ebreak",   0, INSN_CLASS_C,   "",  MATCH_C_EBREAK, MASK_C_EBREAK, match_opcode, 0 },
{"c.jr",       0, INSN_CLASS_C,   "d",  MATCH_C_JR, MASK_C_JR, match_rd_nonzero, INSN_BRANCH },
{"c.jalr",     0, INSN_CLASS_C,   "d",  MATCH_C_JALR, MASK_C_JALR, match_rd_nonzero, INSN_JSR },
{"c.j",        0, INSN_CLASS_C,   "Ca",  MATCH_C_J, MASK_C_J, match_opcode, INSN_BRANCH },
{"c.jal",     32, INSN_CLASS_C, "Ca",  MATCH_C_JAL, MASK_C_JAL, match_opcode, INSN_JSR },
{"c.beqz",     0, INSN_CLASS_C,   "Cs,Cp",  MATCH_C_BEQZ, MASK_C_BEQZ, match_opcode, INSN_CONDBRANCH },
{"c.bnez",     0, INSN_CLASS_C,   "Cs,Cp",  MATCH_C_BNEZ, MASK_C_BNEZ, match_opcode, INSN_CONDBRANCH },
{"c.lwsp",     0, INSN_CLASS_C,   "d,Cm(Cc)",  MATCH_C_LWSP, MASK_C_LWSP, match_rd_nonzero, 0 },
{"c.lw",       0, INSN_CLASS_C,   "Ct,Ck(Cs)",  MATCH_C_LW, MASK_C_LW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.swsp",     0, INSN_CLASS_C,   "CV,CM(Cc)",  MATCH_C_SWSP, MASK_C_SWSP, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.sw",       0, INSN_CLASS_C,   "Ct,Ck(Cs)",  MATCH_C_SW, MASK_C_SW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.nop",      0, INSN_CLASS_C,   "",  MATCH_C_ADDI, 0xffff, match_opcode, INSN_ALIAS },
{"c.nop",      0, INSN_CLASS_C,   "Cj",  MATCH_C_ADDI, MASK_C_ADDI | MASK_RD, match_opcode, INSN_ALIAS },
{"c.mv",       0, INSN_CLASS_C,   "d,CV",  MATCH_C_MV, MASK_C_MV, match_c_add_with_hint, 0 },
{"c.lui",      0, INSN_CLASS_C,   "d,Cu",  MATCH_C_LUI, MASK_C_LUI, match_c_lui_with_hint, 0 },
{"c.li",       0, INSN_CLASS_C,   "d,Co",  MATCH_C_LI, MASK_C_LI, match_opcode, 0 },
{"c.addi4spn", 0, INSN_CLASS_C,   "Ct,Cc,CK", MATCH_C_ADDI4SPN, MASK_C_ADDI4SPN, match_c_addi4spn, 0 },
{"c.addi16sp", 0, INSN_CLASS_C,   "Cc,CL", MATCH_C_ADDI16SP, MASK_C_ADDI16SP, match_c_addi16sp, 0 },
{"c.addi",     0, INSN_CLASS_C,   "d,Co",  MATCH_C_ADDI, MASK_C_ADDI, match_opcode, 0 },
{"c.add",      0, INSN_CLASS_C,   "d,CV",  MATCH_C_ADD, MASK_C_ADD, match_c_add_with_hint, 0 },
{"c.sub",      0, INSN_CLASS_C,   "Cs,Ct",  MATCH_C_SUB, MASK_C_SUB, match_opcode, 0 },
{"c.and",      0, INSN_CLASS_C,   "Cs,Ct",  MATCH_C_AND, MASK_C_AND, match_opcode, 0 },
{"c.or",       0, INSN_CLASS_C,   "Cs,Ct",  MATCH_C_OR, MASK_C_OR, match_opcode, 0 },
{"c.xor",      0, INSN_CLASS_C,   "Cs,Ct",  MATCH_C_XOR, MASK_C_XOR, match_opcode, 0 },
{"c.slli",     0, INSN_CLASS_C,   "d,C>",  MATCH_C_SLLI, MASK_C_SLLI, match_c_slli, 0 },
{"c.srli",     0, INSN_CLASS_C,   "Cs,C>",  MATCH_C_SRLI, MASK_C_SRLI, match_c_slli, 0 },
{"c.srai",     0, INSN_CLASS_C,   "Cs,C>",  MATCH_C_SRAI, MASK_C_SRAI, match_c_slli, 0 },
{"c.slli64",   0, INSN_CLASS_C,   "d",  MATCH_C_SLLI64, MASK_C_SLLI64, match_c_slli64, 0 },
{"c.srli64",   0, INSN_CLASS_C,   "Cs",  MATCH_C_SRLI64, MASK_C_SRLI64, match_c_slli64, 0 },
{"c.srai64",   0, INSN_CLASS_C,   "Cs",  MATCH_C_SRAI64, MASK_C_SRAI64, match_c_slli64, 0 },
{"c.andi",     0, INSN_CLASS_C,   "Cs,Co",  MATCH_C_ANDI, MASK_C_ANDI, match_opcode, 0 },
{"c.addiw",   64, INSN_CLASS_C, "d,Co",  MATCH_C_ADDIW, MASK_C_ADDIW, match_rd_nonzero, 0 },
{"c.addw",    64, INSN_CLASS_C, "Cs,Ct",  MATCH_C_ADDW, MASK_C_ADDW, match_opcode, 0 },
{"c.subw",    64, INSN_CLASS_C, "Cs,Ct",  MATCH_C_SUBW, MASK_C_SUBW, match_opcode, 0 },
{"c.ldsp",    64, INSN_CLASS_C, "d,Cn(Cc)",  MATCH_C_LDSP, MASK_C_LDSP, match_rd_nonzero, INSN_DREF|INSN_8_BYTE },
{"c.ld",      64, INSN_CLASS_C, "Ct,Cl(Cs)",  MATCH_C_LD, MASK_C_LD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.sdsp",    64, INSN_CLASS_C, "CV,CN(Cc)",  MATCH_C_SDSP, MASK_C_SDSP, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.sd",      64, INSN_CLASS_C, "Ct,Cl(Cs)",  MATCH_C_SD, MASK_C_SD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.fldsp",    0, INSN_CLASS_D_AND_C,   "D,Cn(Cc)",  MATCH_C_FLDSP, MASK_C_FLDSP, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.fld",      0, INSN_CLASS_D_AND_C,   "CD,Cl(Cs)",  MATCH_C_FLD, MASK_C_FLD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.fsdsp",    0, INSN_CLASS_D_AND_C,   "CT,CN(Cc)",  MATCH_C_FSDSP, MASK_C_FSDSP, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.fsd",      0, INSN_CLASS_D_AND_C,   "CD,Cl(Cs)",  MATCH_C_FSD, MASK_C_FSD, match_opcode, INSN_DREF|INSN_8_BYTE },
{"c.flwsp",   32, INSN_CLASS_F_AND_C, "D,Cm(Cc)",  MATCH_C_FLWSP, MASK_C_FLWSP, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.flw",     32, INSN_CLASS_F_AND_C, "CD,Ck(Cs)",  MATCH_C_FLW, MASK_C_FLW, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.fswsp",   32, INSN_CLASS_F_AND_C, "CT,CM(Cc)",  MATCH_C_FSWSP, MASK_C_FSWSP, match_opcode, INSN_DREF|INSN_4_BYTE },
{"c.fsw",     32, INSN_CLASS_F_AND_C, "CD,Ck(Cs)",  MATCH_C_FSW, MASK_C_FSW, match_opcode, INSN_DREF|INSN_4_BYTE },

/* Supervisor instructions */
{"csrr",       0, INSN_CLASS_ZICSR,   "d,E",  MATCH_CSRRS, MASK_CSRRS | MASK_RS1, match_opcode, INSN_ALIAS },
{"csrwi",      0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRWI, MASK_CSRRWI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrsi",      0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRSI, MASK_CSRRSI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrci",      0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRCI, MASK_CSRRCI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrw",       0, INSN_CLASS_ZICSR,   "E,s",  MATCH_CSRRW, MASK_CSRRW | MASK_RD, match_opcode, INSN_ALIAS },
{"csrw",       0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRWI, MASK_CSRRWI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrs",       0, INSN_CLASS_ZICSR,   "E,s",  MATCH_CSRRS, MASK_CSRRS | MASK_RD, match_opcode, INSN_ALIAS },
{"csrs",       0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRSI, MASK_CSRRSI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrc",       0, INSN_CLASS_ZICSR,   "E,s",  MATCH_CSRRC, MASK_CSRRC | MASK_RD, match_opcode, INSN_ALIAS },
{"csrc",       0, INSN_CLASS_ZICSR,   "E,Z",  MATCH_CSRRCI, MASK_CSRRCI | MASK_RD, match_opcode, INSN_ALIAS },
{"csrrwi",     0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRWI, MASK_CSRRWI, match_opcode, 0 },
{"csrrsi",     0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRSI, MASK_CSRRSI, match_opcode, 0 },
{"csrrci",     0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRCI, MASK_CSRRCI, match_opcode, 0 },
{"csrrw",      0, INSN_CLASS_ZICSR,   "d,E,s",  MATCH_CSRRW, MASK_CSRRW, match_opcode, 0 },
{"csrrw",      0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRWI, MASK_CSRRWI, match_opcode, INSN_ALIAS },
{"csrrs",      0, INSN_CLASS_ZICSR,   "d,E,s",  MATCH_CSRRS, MASK_CSRRS, match_opcode, 0 },
{"csrrs",      0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRSI, MASK_CSRRSI, match_opcode, INSN_ALIAS },
{"csrrc",      0, INSN_CLASS_ZICSR,   "d,E,s",  MATCH_CSRRC, MASK_CSRRC, match_opcode, 0 },
{"csrrc",      0, INSN_CLASS_ZICSR,   "d,E,Z",  MATCH_CSRRCI, MASK_CSRRCI, match_opcode, INSN_ALIAS },
{"uret",       0, INSN_CLASS_I,   "",     MATCH_URET, MASK_URET, match_opcode, 0 },
{"sret",       0, INSN_CLASS_I,   "",     MATCH_SRET, MASK_SRET, match_opcode, 0 },
{"hret",       0, INSN_CLASS_I,   "",     MATCH_HRET, MASK_HRET, match_opcode, 0 },
{"mret",       0, INSN_CLASS_I,   "",     MATCH_MRET, MASK_MRET, match_opcode, 0 },
{"dret",       0, INSN_CLASS_I,   "",     MATCH_DRET, MASK_DRET, match_opcode, 0 },
{"sfence.vm",  0, INSN_CLASS_I,   "",     MATCH_SFENCE_VM, MASK_SFENCE_VM | MASK_RS1, match_opcode, 0 },
{"sfence.vm",  0, INSN_CLASS_I,   "s",    MATCH_SFENCE_VM, MASK_SFENCE_VM, match_opcode, 0 },
{"sfence.vma", 0, INSN_CLASS_I,   "",     MATCH_SFENCE_VMA, MASK_SFENCE_VMA | MASK_RS1 | MASK_RS2, match_opcode, INSN_ALIAS },
{"sfence.vma", 0, INSN_CLASS_I,   "s",    MATCH_SFENCE_VMA, MASK_SFENCE_VMA | MASK_RS2, match_opcode, INSN_ALIAS },
{"sfence.vma", 0, INSN_CLASS_I,   "s,t",  MATCH_SFENCE_VMA, MASK_SFENCE_VMA, match_opcode, 0 },
{"wfi",        0, INSN_CLASS_I,   "",     MATCH_WFI, MASK_WFI, match_opcode, 0 },

/* Amethyst SIMD Instructions */
{"pwsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUB_B, MASK_PWSUB_B, match_opcode, 0 },
{"pwsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUB_H, MASK_PWSUB_H, match_opcode, 0 },
{"pwsub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUB_W, MASK_PWSUB_W, match_opcode, 0 },
{"pdsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUB_B, MASK_PDSUB_B, match_opcode, 0 },
{"pdsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUB_H, MASK_PDSUB_H, match_opcode, 0 },
{"pdsub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUB_W, MASK_PDSUB_W, match_opcode, 0 },
{"pdsub.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUB_D, MASK_PDSUB_D, match_opcode, 0 },
{"pwsubhu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHU_B, MASK_PWSUBHU_B, match_opcode, 0 },
{"pwsubhu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHU_H, MASK_PWSUBHU_H, match_opcode, 0 },
{"pwsubhu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHU_W, MASK_PWSUBHU_W, match_opcode, 0 },
{"pdsubhu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHU_B, MASK_PDSUBHU_B, match_opcode, 0 },
{"pdsubhu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHU_H, MASK_PDSUBHU_H, match_opcode, 0 },
{"pdsubhu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHU_W, MASK_PDSUBHU_W, match_opcode, 0 },
{"pdsubhu.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHU_D, MASK_PDSUBHU_D, match_opcode, 0 },
{"pwsubss.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBSS_B, MASK_PWSUBSS_B, match_opcode, 0 },
{"pwsubss.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBSS_H, MASK_PWSUBSS_H, match_opcode, 0 },
{"pwsubss.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBSS_W, MASK_PWSUBSS_W, match_opcode, 0 },
{"pdsubss.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBSS_B, MASK_PDSUBSS_B, match_opcode, 0 },
{"pdsubss.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBSS_H, MASK_PDSUBSS_H, match_opcode, 0 },
{"pdsubss.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBSS_W, MASK_PDSUBSS_W, match_opcode, 0 },
{"pdsubss.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBSS_D, MASK_PDSUBSS_D, match_opcode, 0 },
{"pwsubus.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBUS_B, MASK_PWSUBUS_B, match_opcode, 0 },
{"pwsubus.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBUS_H, MASK_PWSUBUS_H, match_opcode, 0 },
{"pwsubus.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBUS_W, MASK_PWSUBUS_W, match_opcode, 0 },
{"pdsubus.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBUS_B, MASK_PDSUBUS_B, match_opcode, 0 },
{"pdsubus.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBUS_H, MASK_PDSUBUS_H, match_opcode, 0 },
{"pdsubus.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBUS_W, MASK_PDSUBUS_W, match_opcode, 0 },
{"pdsubus.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBUS_D, MASK_PDSUBUS_D, match_opcode, 0 },
{"pwsubhs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHS_B, MASK_PWSUBHS_B, match_opcode, 0 },
{"pwsubhs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHS_H, MASK_PWSUBHS_H, match_opcode, 0 },
{"pwsubhs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSUBHS_W, MASK_PWSUBHS_W, match_opcode, 0 },
{"pdsubhs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHS_B, MASK_PDSUBHS_B, match_opcode, 0 },
{"pdsubhs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHS_H, MASK_PDSUBHS_H, match_opcode, 0 },
{"pdsubhs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHS_W, MASK_PDSUBHS_W, match_opcode, 0 },
{"pdsubhs.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSUBHS_D, MASK_PDSUBHS_D, match_opcode, 0 },
{"pwhadd.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWHADD_B, MASK_PWHADD_B, match_opcode, 0 },
{"pwhadd.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWHADD_H, MASK_PWHADD_H, match_opcode, 0 },
{"pdhadd.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHADD_B, MASK_PDHADD_B, match_opcode, 0 },
{"pdhadd.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHADD_H, MASK_PDHADD_H, match_opcode, 0 },
{"pdhadd.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHADD_W, MASK_PDHADD_W, match_opcode, 0 },
{"pwhsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWHSUB_B, MASK_PWHSUB_B, match_opcode, 0 },
{"pwhsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWHSUB_H, MASK_PWHSUB_H, match_opcode, 0 },
{"pdhsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHSUB_B, MASK_PDHSUB_B, match_opcode, 0 },
{"pdhsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHSUB_H, MASK_PDHSUB_H, match_opcode, 0 },
{"pdhsub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDHSUB_W, MASK_PDHSUB_W, match_opcode, 0 },
{"pwslt.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLT_B, MASK_PWSLT_B, match_opcode, 0 },
{"pwslt.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLT_H, MASK_PWSLT_H, match_opcode, 0 },
{"pwslt.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLT_W, MASK_PWSLT_W, match_opcode, 0 },
{"pdslt.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLT_B, MASK_PDSLT_B, match_opcode, 0 },
{"pdslt.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLT_H, MASK_PDSLT_H, match_opcode, 0 },
{"pdslt.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLT_W, MASK_PDSLT_W, match_opcode, 0 },
{"pdslt.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLT_D, MASK_PDSLT_D, match_opcode, 0 },
{"pwsltm.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTM_B, MASK_PWSLTM_B, match_opcode, 0 },
{"pwsltm.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTM_H, MASK_PWSLTM_H, match_opcode, 0 },
{"pwsltm.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTM_W, MASK_PWSLTM_W, match_opcode, 0 },
{"pdsltm.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTM_B, MASK_PDSLTM_B, match_opcode, 0 },
{"pdsltm.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTM_H, MASK_PDSLTM_H, match_opcode, 0 },
{"pdsltm.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTM_W, MASK_PDSLTM_W, match_opcode, 0 },
{"pdsltm.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTM_D, MASK_PDSLTM_D, match_opcode, 0 },
{"pwsltu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTU_B, MASK_PWSLTU_B, match_opcode, 0 },
{"pwsltu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTU_H, MASK_PWSLTU_H, match_opcode, 0 },
{"pwsltu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTU_W, MASK_PWSLTU_W, match_opcode, 0 },
{"pdsltu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTU_B, MASK_PDSLTU_B, match_opcode, 0 },
{"pdsltu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTU_H, MASK_PDSLTU_H, match_opcode, 0 },
{"pdsltu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTU_W, MASK_PDSLTU_W, match_opcode, 0 },
{"pdsltu.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTU_D, MASK_PDSLTU_D, match_opcode, 0 },
{"pwsltum.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTUM_B, MASK_PWSLTUM_B, match_opcode, 0 },
{"pwsltum.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTUM_H, MASK_PWSLTUM_H, match_opcode, 0 },
{"pwsltum.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLTUM_W, MASK_PWSLTUM_W, match_opcode, 0 },
{"pdsltum.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTUM_B, MASK_PDSLTUM_B, match_opcode, 0 },
{"pdsltum.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTUM_H, MASK_PDSLTUM_H, match_opcode, 0 },
{"pdsltum.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTUM_W, MASK_PDSLTUM_W, match_opcode, 0 },
{"pdsltum.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLTUM_D, MASK_PDSLTUM_D, match_opcode, 0 },
{"pwmin.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMIN_B, MASK_PWMIN_B, match_opcode, 0 },
{"pwmin.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMIN_H, MASK_PWMIN_H, match_opcode, 0 },
{"pwmin.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMIN_W, MASK_PWMIN_W, match_opcode, 0 },
{"pdmin.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMIN_B, MASK_PDMIN_B, match_opcode, 0 },
{"pdmin.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMIN_H, MASK_PDMIN_H, match_opcode, 0 },
{"pdmin.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMIN_W, MASK_PDMIN_W, match_opcode, 0 },
{"pdmin.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMIN_D, MASK_PDMIN_D, match_opcode, 0 },
{"pwminu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMINU_B, MASK_PWMINU_B, match_opcode, 0 },
{"pwminu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMINU_H, MASK_PWMINU_H, match_opcode, 0 },
{"pwminu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMINU_W, MASK_PWMINU_W, match_opcode, 0 },
{"pdminu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMINU_B, MASK_PDMINU_B, match_opcode, 0 },
{"pdminu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMINU_H, MASK_PDMINU_H, match_opcode, 0 },
{"pdminu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMINU_W, MASK_PDMINU_W, match_opcode, 0 },
{"pdminu.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMINU_D, MASK_PDMINU_D, match_opcode, 0 },
{"pwmax.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAX_B, MASK_PWMAX_B, match_opcode, 0 },
{"pwmax.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAX_H, MASK_PWMAX_H, match_opcode, 0 },
{"pwmax.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAX_W, MASK_PWMAX_W, match_opcode, 0 },
{"pdmax.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAX_B, MASK_PDMAX_B, match_opcode, 0 },
{"pdmax.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAX_H, MASK_PDMAX_H, match_opcode, 0 },
{"pdmax.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAX_W, MASK_PDMAX_W, match_opcode, 0 },
{"pdmax.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAX_D, MASK_PDMAX_D, match_opcode, 0 },
{"pwmaxu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAXU_B, MASK_PWMAXU_B, match_opcode, 0 },
{"pwmaxu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAXU_H, MASK_PWMAXU_H, match_opcode, 0 },
{"pwmaxu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMAXU_W, MASK_PWMAXU_W, match_opcode, 0 },
{"pdmaxu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAXU_B, MASK_PDMAXU_B, match_opcode, 0 },
{"pdmaxu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAXU_H, MASK_PDMAXU_H, match_opcode, 0 },
{"pdmaxu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAXU_W, MASK_PDMAXU_W, match_opcode, 0 },
{"pdmaxu.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMAXU_D, MASK_PDMAXU_D, match_opcode, 0 },
{"pwadd.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADD_B, MASK_PWADD_B, match_opcode, 0 },
{"pwadd.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADD_H, MASK_PWADD_H, match_opcode, 0 },
{"pwadd.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADD_W, MASK_PWADD_W, match_opcode, 0 },
{"pdadd.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADD_B, MASK_PDADD_B, match_opcode, 0 },
{"pdadd.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADD_H, MASK_PDADD_H, match_opcode, 0 },
{"pdadd.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADD_W, MASK_PDADD_W, match_opcode, 0 },
{"pdadd.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADD_D, MASK_PDADD_D, match_opcode, 0 },
{"pwavgu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGU_B, MASK_PWAVGU_B, match_opcode, 0 },
{"pwavgu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGU_H, MASK_PWAVGU_H, match_opcode, 0 },
{"pwavgu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGU_W, MASK_PWAVGU_W, match_opcode, 0 },
{"pdavgu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGU_B, MASK_PDAVGU_B, match_opcode, 0 },
{"pdavgu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGU_H, MASK_PDAVGU_H, match_opcode, 0 },
{"pdavgu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGU_W, MASK_PDAVGU_W, match_opcode, 0 },
{"pdavgu.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGU_D, MASK_PDAVGU_D, match_opcode, 0 },
{"pwaddss.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDSS_B, MASK_PWADDSS_B, match_opcode, 0 },
{"pwaddss.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDSS_H, MASK_PWADDSS_H, match_opcode, 0 },
{"pwaddss.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDSS_W, MASK_PWADDSS_W, match_opcode, 0 },
{"pdaddss.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDSS_B, MASK_PDADDSS_B, match_opcode, 0 },
{"pdaddss.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDSS_H, MASK_PDADDSS_H, match_opcode, 0 },
{"pdaddss.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDSS_W, MASK_PDADDSS_W, match_opcode, 0 },
{"pdaddss.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDSS_D, MASK_PDADDSS_D, match_opcode, 0 },
{"pwaddus.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDUS_B, MASK_PWADDUS_B, match_opcode, 0 },
{"pwaddus.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDUS_H, MASK_PWADDUS_H, match_opcode, 0 },
{"pwaddus.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWADDUS_W, MASK_PWADDUS_W, match_opcode, 0 },
{"pdaddus.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDUS_B, MASK_PDADDUS_B, match_opcode, 0 },
{"pdaddus.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDUS_H, MASK_PDADDUS_H, match_opcode, 0 },
{"pdaddus.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDUS_W, MASK_PDADDUS_W, match_opcode, 0 },
{"pdaddus.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDADDUS_D, MASK_PDADDUS_D, match_opcode, 0 },
{"pwavgs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGS_B, MASK_PWAVGS_B, match_opcode, 0 },
{"pwavgs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGS_H, MASK_PWAVGS_H, match_opcode, 0 },
{"pwavgs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAVGS_W, MASK_PWAVGS_W, match_opcode, 0 },
{"pdavgs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGS_B, MASK_PDAVGS_B, match_opcode, 0 },
{"pdavgs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGS_H, MASK_PDAVGS_H, match_opcode, 0 },
{"pdavgs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGS_W, MASK_PDAVGS_W, match_opcode, 0 },
{"pdavgs.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAVGS_D, MASK_PDAVGS_D, match_opcode, 0 },
{"pwsh1add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH1ADD_B, MASK_PWSH1ADD_B, match_opcode, 0 },
{"pwsh1add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH1ADD_H, MASK_PWSH1ADD_H, match_opcode, 0 },
{"pwsh1add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH1ADD_W, MASK_PWSH1ADD_W, match_opcode, 0 },
{"pdsh1add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH1ADD_B, MASK_PDSH1ADD_B, match_opcode, 0 },
{"pdsh1add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH1ADD_H, MASK_PDSH1ADD_H, match_opcode, 0 },
{"pdsh1add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH1ADD_W, MASK_PDSH1ADD_W, match_opcode, 0 },
{"pdsh1add.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH1ADD_D, MASK_PDSH1ADD_D, match_opcode, 0 },
{"pwsh2add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH2ADD_B, MASK_PWSH2ADD_B, match_opcode, 0 },
{"pwsh2add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH2ADD_H, MASK_PWSH2ADD_H, match_opcode, 0 },
{"pwsh2add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH2ADD_W, MASK_PWSH2ADD_W, match_opcode, 0 },
{"pdsh2add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH2ADD_B, MASK_PDSH2ADD_B, match_opcode, 0 },
{"pdsh2add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH2ADD_H, MASK_PDSH2ADD_H, match_opcode, 0 },
{"pdsh2add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH2ADD_W, MASK_PDSH2ADD_W, match_opcode, 0 },
{"pdsh2add.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH2ADD_D, MASK_PDSH2ADD_D, match_opcode, 0 },
{"pwsh3add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH3ADD_B, MASK_PWSH3ADD_B, match_opcode, 0 },
{"pwsh3add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH3ADD_H, MASK_PWSH3ADD_H, match_opcode, 0 },
{"pwsh3add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSH3ADD_W, MASK_PWSH3ADD_W, match_opcode, 0 },
{"pdsh3add.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH3ADD_B, MASK_PDSH3ADD_B, match_opcode, 0 },
{"pdsh3add.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH3ADD_H, MASK_PDSH3ADD_H, match_opcode, 0 },
{"pdsh3add.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH3ADD_W, MASK_PDSH3ADD_W, match_opcode, 0 },
{"pdsh3add.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSH3ADD_D, MASK_PDSH3ADD_D, match_opcode, 0 },
{"pwand", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWAND, MASK_PWAND, match_opcode, 0 },
{"pdand", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDAND, MASK_PDAND, match_opcode, 0 },
{"pwor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWOR, MASK_PWOR, match_opcode, 0 },
{"pdor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDOR, MASK_PDOR, match_opcode, 0 },
{"pwxor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWXOR, MASK_PWXOR, match_opcode, 0 },
{"pdxor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDXOR, MASK_PDXOR, match_opcode, 0 },
{"pwandn", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWANDN, MASK_PWANDN, match_opcode, 0 },
{"pdandn", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDANDN, MASK_PDANDN, match_opcode, 0 },
{"pworn", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWORN, MASK_PWORN, match_opcode, 0 },
{"pdorn", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDORN, MASK_PDORN, match_opcode, 0 },
{"pwxnor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWXNOR, MASK_PWXNOR, match_opcode, 0 },
{"pdxnor", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDXNOR, MASK_PDXNOR, match_opcode, 0 },
{"pwsll.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLL_B, MASK_PWSLL_B, match_opcode, 0 },
{"pwsll.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLL_H, MASK_PWSLL_H, match_opcode, 0 },
{"pwsll.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLL_W, MASK_PWSLL_W, match_opcode, 0 },
{"pdsll.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLL_B, MASK_PDSLL_B, match_opcode, 0 },
{"pdsll.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLL_H, MASK_PDSLL_H, match_opcode, 0 },
{"pdsll.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLL_W, MASK_PDSLL_W, match_opcode, 0 },
{"pdsll.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLL_D, MASK_PDSLL_D, match_opcode, 0 },
{"pwsrl.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRL_B, MASK_PWSRL_B, match_opcode, 0 },
{"pwsrl.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRL_H, MASK_PWSRL_H, match_opcode, 0 },
{"pwsrl.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRL_W, MASK_PWSRL_W, match_opcode, 0 },
{"pdsrl.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRL_B, MASK_PDSRL_B, match_opcode, 0 },
{"pdsrl.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRL_H, MASK_PDSRL_H, match_opcode, 0 },
{"pdsrl.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRL_W, MASK_PDSRL_W, match_opcode, 0 },
{"pdsrl.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRL_D, MASK_PDSRL_D, match_opcode, 0 },
{"pwslo.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLO_B, MASK_PWSLO_B, match_opcode, 0 },
{"pwslo.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLO_H, MASK_PWSLO_H, match_opcode, 0 },
{"pwslo.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSLO_W, MASK_PWSLO_W, match_opcode, 0 },
{"pdslo.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLO_B, MASK_PDSLO_B, match_opcode, 0 },
{"pdslo.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLO_H, MASK_PDSLO_H, match_opcode, 0 },
{"pdslo.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLO_W, MASK_PDSLO_W, match_opcode, 0 },
{"pdslo.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSLO_D, MASK_PDSLO_D, match_opcode, 0 },
{"pwsro.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRO_B, MASK_PWSRO_B, match_opcode, 0 },
{"pwsro.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRO_H, MASK_PWSRO_H, match_opcode, 0 },
{"pwsro.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRO_W, MASK_PWSRO_W, match_opcode, 0 },
{"pdsro.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRO_B, MASK_PDSRO_B, match_opcode, 0 },
{"pdsro.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRO_H, MASK_PDSRO_H, match_opcode, 0 },
{"pdsro.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRO_W, MASK_PDSRO_W, match_opcode, 0 },
{"pdsro.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRO_D, MASK_PDSRO_D, match_opcode, 0 },
{"pwsra.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRA_B, MASK_PWSRA_B, match_opcode, 0 },
{"pwsra.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRA_H, MASK_PWSRA_H, match_opcode, 0 },
{"pwsra.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWSRA_W, MASK_PWSRA_W, match_opcode, 0 },
{"pdsra.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRA_B, MASK_PDSRA_B, match_opcode, 0 },
{"pdsra.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRA_H, MASK_PDSRA_H, match_opcode, 0 },
{"pdsra.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRA_W, MASK_PDSRA_W, match_opcode, 0 },
{"pdsra.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDSRA_D, MASK_PDSRA_D, match_opcode, 0 },
{"pwrol.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROL_B, MASK_PWROL_B, match_opcode, 0 },
{"pwrol.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROL_H, MASK_PWROL_H, match_opcode, 0 },
{"pwrol.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROL_W, MASK_PWROL_W, match_opcode, 0 },
{"pdrol.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROL_B, MASK_PDROL_B, match_opcode, 0 },
{"pdrol.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROL_H, MASK_PDROL_H, match_opcode, 0 },
{"pdrol.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROL_W, MASK_PDROL_W, match_opcode, 0 },
{"pdrol.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROL_D, MASK_PDROL_D, match_opcode, 0 },
{"pwror.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROR_B, MASK_PWROR_B, match_opcode, 0 },
{"pwror.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROR_H, MASK_PWROR_H, match_opcode, 0 },
{"pwror.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWROR_W, MASK_PWROR_W, match_opcode, 0 },
{"pdror.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROR_B, MASK_PDROR_B, match_opcode, 0 },
{"pdror.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROR_H, MASK_PDROR_H, match_opcode, 0 },
{"pdror.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROR_W, MASK_PDROR_W, match_opcode, 0 },
{"pdror.d", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDROR_D, MASK_PDROR_D, match_opcode, 0 },
{"pwmulssb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSB_B, MASK_PWMULSSB_B, match_opcode, 0 },
{"pwmulssb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSB_H, MASK_PWMULSSB_H, match_opcode, 0 },
{"pwmulssb.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSB_W, MASK_PWMULSSB_W, match_opcode, 0 },
{"pdmulssb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSB_B, MASK_PDMULSSB_B, match_opcode, 0 },
{"pdmulssb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSB_H, MASK_PDMULSSB_H, match_opcode, 0 },
{"pdmulssb.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSB_W, MASK_PDMULSSB_W, match_opcode, 0 },
{"pwmulsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUB_B, MASK_PWMULSUB_B, match_opcode, 0 },
{"pwmulsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUB_H, MASK_PWMULSUB_H, match_opcode, 0 },
{"pwmulsub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUB_W, MASK_PWMULSUB_W, match_opcode, 0 },
{"pdmulsub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUB_B, MASK_PDMULSUB_B, match_opcode, 0 },
{"pdmulsub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUB_H, MASK_PDMULSUB_H, match_opcode, 0 },
{"pdmulsub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUB_W, MASK_PDMULSUB_W, match_opcode, 0 },
{"pwmuluub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUB_B, MASK_PWMULUUB_B, match_opcode, 0 },
{"pwmuluub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUB_H, MASK_PWMULUUB_H, match_opcode, 0 },
{"pwmuluub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUB_W, MASK_PWMULUUB_W, match_opcode, 0 },
{"pdmuluub.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUB_B, MASK_PDMULUUB_B, match_opcode, 0 },
{"pdmuluub.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUB_H, MASK_PDMULUUB_H, match_opcode, 0 },
{"pdmuluub.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUB_W, MASK_PDMULUUB_W, match_opcode, 0 },
{"pwmulccb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCB_B, MASK_PWMULCCB_B, match_opcode, 0 },
{"pwmulccb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCB_H, MASK_PWMULCCB_H, match_opcode, 0 },
{"pdmulccb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCB_B, MASK_PDMULCCB_B, match_opcode, 0 },
{"pdmulccb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCB_H, MASK_PDMULCCB_H, match_opcode, 0 },
{"pdmulccb.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCB_W, MASK_PDMULCCB_W, match_opcode, 0 },
{"pwmulssbqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQU_B, MASK_PWMULSSBQU_B, match_opcode, 0 },
{"pwmulssbqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQU_H, MASK_PWMULSSBQU_H, match_opcode, 0 },
{"pwmulssbqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQU_W, MASK_PWMULSSBQU_W, match_opcode, 0 },
{"pdmulssbqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQU_B, MASK_PDMULSSBQU_B, match_opcode, 0 },
{"pdmulssbqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQU_H, MASK_PDMULSSBQU_H, match_opcode, 0 },
{"pdmulssbqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQU_W, MASK_PDMULSSBQU_W, match_opcode, 0 },
{"pwmulsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQU_B, MASK_PWMULSUBQU_B, match_opcode, 0 },
{"pwmulsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQU_H, MASK_PWMULSUBQU_H, match_opcode, 0 },
{"pwmulsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQU_W, MASK_PWMULSUBQU_W, match_opcode, 0 },
{"pdmulsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQU_B, MASK_PDMULSUBQU_B, match_opcode, 0 },
{"pdmulsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQU_H, MASK_PDMULSUBQU_H, match_opcode, 0 },
{"pdmulsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQU_W, MASK_PDMULSUBQU_W, match_opcode, 0 },
{"pwmuluubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQU_B, MASK_PWMULUUBQU_B, match_opcode, 0 },
{"pwmuluubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQU_H, MASK_PWMULUUBQU_H, match_opcode, 0 },
{"pwmuluubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQU_W, MASK_PWMULUUBQU_W, match_opcode, 0 },
{"pdmuluubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQU_B, MASK_PDMULUUBQU_B, match_opcode, 0 },
{"pdmuluubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQU_H, MASK_PDMULUUBQU_H, match_opcode, 0 },
{"pdmuluubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQU_W, MASK_PDMULUUBQU_W, match_opcode, 0 },
{"pwmulssbqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQS_B, MASK_PWMULSSBQS_B, match_opcode, 0 },
{"pwmulssbqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQS_H, MASK_PWMULSSBQS_H, match_opcode, 0 },
{"pwmulssbqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSBQS_W, MASK_PWMULSSBQS_W, match_opcode, 0 },
{"pdmulssbqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQS_B, MASK_PDMULSSBQS_B, match_opcode, 0 },
{"pdmulssbqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQS_H, MASK_PDMULSSBQS_H, match_opcode, 0 },
{"pdmulssbqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSSBQS_W, MASK_PDMULSSBQS_W, match_opcode, 0 },
{"pwmulsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQS_B, MASK_PWMULSUBQS_B, match_opcode, 0 },
{"pwmulsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQS_H, MASK_PWMULSUBQS_H, match_opcode, 0 },
{"pwmulsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUBQS_W, MASK_PWMULSUBQS_W, match_opcode, 0 },
{"pdmulsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQS_B, MASK_PDMULSUBQS_B, match_opcode, 0 },
{"pdmulsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQS_H, MASK_PDMULSUBQS_H, match_opcode, 0 },
{"pdmulsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUBQS_W, MASK_PDMULSUBQS_W, match_opcode, 0 },
{"pwmuluubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQS_B, MASK_PWMULUUBQS_B, match_opcode, 0 },
{"pwmuluubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQS_H, MASK_PWMULUUBQS_H, match_opcode, 0 },
{"pwmuluubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUBQS_W, MASK_PWMULUUBQS_W, match_opcode, 0 },
{"pdmuluubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQS_B, MASK_PDMULUUBQS_B, match_opcode, 0 },
{"pdmuluubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQS_H, MASK_PDMULUUBQS_H, match_opcode, 0 },
{"pdmuluubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUBQS_W, MASK_PDMULUUBQS_W, match_opcode, 0 },
{"pwmulccbq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCBQ_B, MASK_PWMULCCBQ_B, match_opcode, 0 },
{"pwmulccbq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCBQ_H, MASK_PWMULCCBQ_H, match_opcode, 0 },
{"pdmulccbq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCBQ_B, MASK_PDMULCCBQ_B, match_opcode, 0 },
{"pdmulccbq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCBQ_H, MASK_PDMULCCBQ_H, match_opcode, 0 },
{"pdmulccbq.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCBQ_W, MASK_PDMULCCBQ_W, match_opcode, 0 },
{"pwmulsst.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSST_B, MASK_PWMULSST_B, match_opcode, 0 },
{"pwmulsst.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSST_H, MASK_PWMULSST_H, match_opcode, 0 },
{"pwmulsst.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSST_W, MASK_PWMULSST_W, match_opcode, 0 },
{"pdmulsst.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSST_B, MASK_PDMULSST_B, match_opcode, 0 },
{"pdmulsst.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSST_H, MASK_PDMULSST_H, match_opcode, 0 },
{"pdmulsst.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSST_W, MASK_PDMULSST_W, match_opcode, 0 },
{"pwmulsut.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUT_B, MASK_PWMULSUT_B, match_opcode, 0 },
{"pwmulsut.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUT_H, MASK_PWMULSUT_H, match_opcode, 0 },
{"pwmulsut.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUT_W, MASK_PWMULSUT_W, match_opcode, 0 },
{"pdmulsut.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUT_B, MASK_PDMULSUT_B, match_opcode, 0 },
{"pdmulsut.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUT_H, MASK_PDMULSUT_H, match_opcode, 0 },
{"pdmulsut.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULSUT_W, MASK_PDMULSUT_W, match_opcode, 0 },
{"pwmuluut.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUT_B, MASK_PWMULUUT_B, match_opcode, 0 },
{"pwmuluut.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUT_H, MASK_PWMULUUT_H, match_opcode, 0 },
{"pwmuluut.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUT_W, MASK_PWMULUUT_W, match_opcode, 0 },
{"pdmuluut.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUT_B, MASK_PDMULUUT_B, match_opcode, 0 },
{"pdmuluut.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUT_H, MASK_PDMULUUT_H, match_opcode, 0 },
{"pdmuluut.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULUUT_W, MASK_PDMULUUT_W, match_opcode, 0 },
{"pwmulcct.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCT_B, MASK_PWMULCCT_B, match_opcode, 0 },
{"pwmulcct.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCT_H, MASK_PWMULCCT_H, match_opcode, 0 },
{"pdmulcct.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCT_B, MASK_PDMULCCT_B, match_opcode, 0 },
{"pdmulcct.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCT_H, MASK_PDMULCCT_H, match_opcode, 0 },
{"pdmulcct.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCCT_W, MASK_PDMULCCT_W, match_opcode, 0 },
{"pwmulssw.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSW_B, MASK_PWMULSSW_B, match_opcode, 0 },
{"pwmulssw.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSW_H, MASK_PWMULSSW_H, match_opcode, 0 },
{"pwmulssw.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSSW_W, MASK_PWMULSSW_W, match_opcode, 0 },
{"pwmulsuw.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUW_B, MASK_PWMULSUW_B, match_opcode, 0 },
{"pwmulsuw.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUW_H, MASK_PWMULSUW_H, match_opcode, 0 },
{"pwmulsuw.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULSUW_W, MASK_PWMULSUW_W, match_opcode, 0 },
{"pwmuluuw.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUW_B, MASK_PWMULUUW_B, match_opcode, 0 },
{"pwmuluuw.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUW_H, MASK_PWMULUUW_H, match_opcode, 0 },
{"pwmuluuw.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULUUW_W, MASK_PWMULUUW_W, match_opcode, 0 },
{"pwmulccw.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCW_B, MASK_PWMULCCW_B, match_opcode, 0 },
{"pwmulccw.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCCW_H, MASK_PWMULCCW_H, match_opcode, 0 },
{"pwmulcrw.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRW_B, MASK_PWMULCRW_B, match_opcode, 0 },
{"pwmulcrw.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRW_H, MASK_PWMULCRW_H, match_opcode, 0 },
{"pwmulcrbq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRBQ_B, MASK_PWMULCRBQ_B, match_opcode, 0 },
{"pwmulcrbq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRBQ_H, MASK_PWMULCRBQ_H, match_opcode, 0 },
{"pdmulcrbq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRBQ_B, MASK_PDMULCRBQ_B, match_opcode, 0 },
{"pdmulcrbq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRBQ_H, MASK_PDMULCRBQ_H, match_opcode, 0 },
{"pdmulcrbq.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRBQ_W, MASK_PDMULCRBQ_W, match_opcode, 0 },
{"pwmulcrb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRB_B, MASK_PWMULCRB_B, match_opcode, 0 },
{"pwmulcrb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRB_H, MASK_PWMULCRB_H, match_opcode, 0 },
{"pdmulcrb.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRB_B, MASK_PDMULCRB_B, match_opcode, 0 },
{"pdmulcrb.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRB_H, MASK_PDMULCRB_H, match_opcode, 0 },
{"pdmulcrb.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRB_W, MASK_PDMULCRB_W, match_opcode, 0 },
{"pwmulcrt.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRT_B, MASK_PWMULCRT_B, match_opcode, 0 },
{"pwmulcrt.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMULCRT_H, MASK_PWMULCRT_H, match_opcode, 0 },
{"pdmulcrt.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRT_B, MASK_PDMULCRT_B, match_opcode, 0 },
{"pdmulcrt.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRT_H, MASK_PDMULCRT_H, match_opcode, 0 },
{"pdmulcrt.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMULCRT_W, MASK_PDMULCRT_W, match_opcode, 0 },
{"crot.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CROT_H, MASK_CROT_H, match_opcode, 0 },
{"crot.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CROT_W, MASK_CROT_W, match_opcode, 0 },
{"cmagarg.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CMAGARG_H, MASK_CMAGARG_H, match_opcode, 0 },
{"cmagarg.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CMAGARG_W, MASK_CMAGARG_W, match_opcode, 0 },
{"chrot.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CHROT_H, MASK_CHROT_H, match_opcode, 0 },
{"chrot.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CHROT_W, MASK_CHROT_W, match_opcode, 0 },
{"chmagarg.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CHMAGARG_H, MASK_CHMAGARG_H, match_opcode, 0 },
{"chmagarg.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_CHMAGARG_W, MASK_CHMAGARG_W, match_opcode, 0 },
{"pwzmaccs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCS_B, MASK_PWZMACCS_B, match_opcode, 0 },
{"pwzmaccs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCS_H, MASK_PWZMACCS_H, match_opcode, 0 },
{"pwzmaccs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCS_W, MASK_PWZMACCS_W, match_opcode, 0 },
{"pdzmaccs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCS_B, MASK_PDZMACCS_B, match_opcode, 0 },
{"pdzmaccs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCS_H, MASK_PDZMACCS_H, match_opcode, 0 },
{"pdzmaccs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCS_W, MASK_PDZMACCS_W, match_opcode, 0 },
{"pwzmaccqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQS_B, MASK_PWZMACCQS_B, match_opcode, 0 },
{"pwzmaccqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQS_H, MASK_PWZMACCQS_H, match_opcode, 0 },
{"pwzmaccqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQS_W, MASK_PWZMACCQS_W, match_opcode, 0 },
{"pdzmaccqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQS_B, MASK_PDZMACCQS_B, match_opcode, 0 },
{"pdzmaccqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQS_H, MASK_PDZMACCQS_H, match_opcode, 0 },
{"pdzmaccqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQS_W, MASK_PDZMACCQS_W, match_opcode, 0 },
{"pwzmaccu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCU_B, MASK_PWZMACCU_B, match_opcode, 0 },
{"pwzmaccu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCU_H, MASK_PWZMACCU_H, match_opcode, 0 },
{"pwzmaccu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCU_W, MASK_PWZMACCU_W, match_opcode, 0 },
{"pdzmaccu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCU_B, MASK_PDZMACCU_B, match_opcode, 0 },
{"pdzmaccu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCU_H, MASK_PDZMACCU_H, match_opcode, 0 },
{"pdzmaccu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCU_W, MASK_PDZMACCU_W, match_opcode, 0 },
{"pwzmaccqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQU_B, MASK_PWZMACCQU_B, match_opcode, 0 },
{"pwzmaccqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQU_H, MASK_PWZMACCQU_H, match_opcode, 0 },
{"pwzmaccqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCQU_W, MASK_PWZMACCQU_W, match_opcode, 0 },
{"pdzmaccqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQU_B, MASK_PDZMACCQU_B, match_opcode, 0 },
{"pdzmaccqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQU_H, MASK_PDZMACCQU_H, match_opcode, 0 },
{"pdzmaccqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCQU_W, MASK_PDZMACCQU_W, match_opcode, 0 },
{"pwzmacccc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCC_B, MASK_PWZMACCCC_B, match_opcode, 0 },
{"pwzmacccc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCC_H, MASK_PWZMACCCC_H, match_opcode, 0 },
{"pdzmacccc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCC_B, MASK_PDZMACCCC_B, match_opcode, 0 },
{"pdzmacccc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCC_H, MASK_PDZMACCCC_H, match_opcode, 0 },
{"pwzmaccccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCCQ_B, MASK_PWZMACCCCQ_B, match_opcode, 0 },
{"pwzmaccccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCCQ_H, MASK_PWZMACCCCQ_H, match_opcode, 0 },
{"pdzmaccccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCCQ_B, MASK_PDZMACCCCQ_B, match_opcode, 0 },
{"pdzmaccccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCCQ_H, MASK_PDZMACCCCQ_H, match_opcode, 0 },
{"pwzmacccr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCR_B, MASK_PWZMACCCR_B, match_opcode, 0 },
{"pwzmacccr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCR_H, MASK_PWZMACCCR_H, match_opcode, 0 },
{"pdzmacccr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCR_B, MASK_PDZMACCCR_B, match_opcode, 0 },
{"pdzmacccr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCR_H, MASK_PDZMACCCR_H, match_opcode, 0 },
{"pwzmacccrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCRQ_B, MASK_PWZMACCCRQ_B, match_opcode, 0 },
{"pwzmacccrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMACCCRQ_H, MASK_PWZMACCCRQ_H, match_opcode, 0 },
{"pdzmacccrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCRQ_B, MASK_PDZMACCCRQ_B, match_opcode, 0 },
{"pdzmacccrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMACCCRQ_H, MASK_PDZMACCCRQ_H, match_opcode, 0 },
{"pwzmsubs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBS_B, MASK_PWZMSUBS_B, match_opcode, 0 },
{"pwzmsubs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBS_H, MASK_PWZMSUBS_H, match_opcode, 0 },
{"pwzmsubs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBS_W, MASK_PWZMSUBS_W, match_opcode, 0 },
{"pdzmsubs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBS_B, MASK_PDZMSUBS_B, match_opcode, 0 },
{"pdzmsubs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBS_H, MASK_PDZMSUBS_H, match_opcode, 0 },
{"pdzmsubs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBS_W, MASK_PDZMSUBS_W, match_opcode, 0 },
{"pwzmsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQS_B, MASK_PWZMSUBQS_B, match_opcode, 0 },
{"pwzmsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQS_H, MASK_PWZMSUBQS_H, match_opcode, 0 },
{"pwzmsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQS_W, MASK_PWZMSUBQS_W, match_opcode, 0 },
{"pdzmsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQS_B, MASK_PDZMSUBQS_B, match_opcode, 0 },
{"pdzmsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQS_H, MASK_PDZMSUBQS_H, match_opcode, 0 },
{"pdzmsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQS_W, MASK_PDZMSUBQS_W, match_opcode, 0 },
{"pwzmsubu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBU_B, MASK_PWZMSUBU_B, match_opcode, 0 },
{"pwzmsubu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBU_H, MASK_PWZMSUBU_H, match_opcode, 0 },
{"pwzmsubu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBU_W, MASK_PWZMSUBU_W, match_opcode, 0 },
{"pdzmsubu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBU_B, MASK_PDZMSUBU_B, match_opcode, 0 },
{"pdzmsubu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBU_H, MASK_PDZMSUBU_H, match_opcode, 0 },
{"pdzmsubu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBU_W, MASK_PDZMSUBU_W, match_opcode, 0 },
{"pwzmsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQU_B, MASK_PWZMSUBQU_B, match_opcode, 0 },
{"pwzmsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQU_H, MASK_PWZMSUBQU_H, match_opcode, 0 },
{"pwzmsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBQU_W, MASK_PWZMSUBQU_W, match_opcode, 0 },
{"pdzmsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQU_B, MASK_PDZMSUBQU_B, match_opcode, 0 },
{"pdzmsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQU_H, MASK_PDZMSUBQU_H, match_opcode, 0 },
{"pdzmsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBQU_W, MASK_PDZMSUBQU_W, match_opcode, 0 },
{"pwzmsubcc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCC_B, MASK_PWZMSUBCC_B, match_opcode, 0 },
{"pwzmsubcc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCC_H, MASK_PWZMSUBCC_H, match_opcode, 0 },
{"pdzmsubcc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCC_B, MASK_PDZMSUBCC_B, match_opcode, 0 },
{"pdzmsubcc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCC_H, MASK_PDZMSUBCC_H, match_opcode, 0 },
{"pwzmsubccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCCQ_B, MASK_PWZMSUBCCQ_B, match_opcode, 0 },
{"pwzmsubccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCCQ_H, MASK_PWZMSUBCCQ_H, match_opcode, 0 },
{"pdzmsubccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCCQ_B, MASK_PDZMSUBCCQ_B, match_opcode, 0 },
{"pdzmsubccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCCQ_H, MASK_PDZMSUBCCQ_H, match_opcode, 0 },
{"pwzmsubcr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCR_B, MASK_PWZMSUBCR_B, match_opcode, 0 },
{"pwzmsubcr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCR_H, MASK_PWZMSUBCR_H, match_opcode, 0 },
{"pdzmsubcr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCR_B, MASK_PDZMSUBCR_B, match_opcode, 0 },
{"pdzmsubcr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCR_H, MASK_PDZMSUBCR_H, match_opcode, 0 },
{"pwzmsubcrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCRQ_B, MASK_PWZMSUBCRQ_B, match_opcode, 0 },
{"pwzmsubcrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWZMSUBCRQ_H, MASK_PWZMSUBCRQ_H, match_opcode, 0 },
{"pdzmsubcrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCRQ_B, MASK_PDZMSUBCRQ_B, match_opcode, 0 },
{"pdzmsubcrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDZMSUBCRQ_H, MASK_PDZMSUBCRQ_H, match_opcode, 0 },
{"pwmaccs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCS_B, MASK_PWMACCS_B, match_opcode, 0 },
{"pwmaccs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCS_H, MASK_PWMACCS_H, match_opcode, 0 },
{"pwmaccs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCS_W, MASK_PWMACCS_W, match_opcode, 0 },
{"pdmaccs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCS_B, MASK_PDMACCS_B, match_opcode, 0 },
{"pdmaccs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCS_H, MASK_PDMACCS_H, match_opcode, 0 },
{"pdmaccs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCS_W, MASK_PDMACCS_W, match_opcode, 0 },
{"pwmaccqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQS_B, MASK_PWMACCQS_B, match_opcode, 0 },
{"pwmaccqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQS_H, MASK_PWMACCQS_H, match_opcode, 0 },
{"pwmaccqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQS_W, MASK_PWMACCQS_W, match_opcode, 0 },
{"pdmaccqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQS_B, MASK_PDMACCQS_B, match_opcode, 0 },
{"pdmaccqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQS_H, MASK_PDMACCQS_H, match_opcode, 0 },
{"pdmaccqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQS_W, MASK_PDMACCQS_W, match_opcode, 0 },
{"pwmaccu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCU_B, MASK_PWMACCU_B, match_opcode, 0 },
{"pwmaccu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCU_H, MASK_PWMACCU_H, match_opcode, 0 },
{"pwmaccu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCU_W, MASK_PWMACCU_W, match_opcode, 0 },
{"pdmaccu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCU_B, MASK_PDMACCU_B, match_opcode, 0 },
{"pdmaccu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCU_H, MASK_PDMACCU_H, match_opcode, 0 },
{"pdmaccu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCU_W, MASK_PDMACCU_W, match_opcode, 0 },
{"pwmaccqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQU_B, MASK_PWMACCQU_B, match_opcode, 0 },
{"pwmaccqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQU_H, MASK_PWMACCQU_H, match_opcode, 0 },
{"pwmaccqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCQU_W, MASK_PWMACCQU_W, match_opcode, 0 },
{"pdmaccqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQU_B, MASK_PDMACCQU_B, match_opcode, 0 },
{"pdmaccqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQU_H, MASK_PDMACCQU_H, match_opcode, 0 },
{"pdmaccqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCQU_W, MASK_PDMACCQU_W, match_opcode, 0 },
{"pwmacccc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCC_B, MASK_PWMACCCC_B, match_opcode, 0 },
{"pwmacccc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCC_H, MASK_PWMACCCC_H, match_opcode, 0 },
{"pdmacccc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCC_B, MASK_PDMACCCC_B, match_opcode, 0 },
{"pdmacccc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCC_H, MASK_PDMACCCC_H, match_opcode, 0 },
{"pwmaccccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCCQ_B, MASK_PWMACCCCQ_B, match_opcode, 0 },
{"pwmaccccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCCQ_H, MASK_PWMACCCCQ_H, match_opcode, 0 },
{"pdmaccccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCCQ_B, MASK_PDMACCCCQ_B, match_opcode, 0 },
{"pdmaccccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCCQ_H, MASK_PDMACCCCQ_H, match_opcode, 0 },
{"pwmacccr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCR_B, MASK_PWMACCCR_B, match_opcode, 0 },
{"pwmacccr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCR_H, MASK_PWMACCCR_H, match_opcode, 0 },
{"pdmacccr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCR_B, MASK_PDMACCCR_B, match_opcode, 0 },
{"pdmacccr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCR_H, MASK_PDMACCCR_H, match_opcode, 0 },
{"pwmacccrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCRQ_B, MASK_PWMACCCRQ_B, match_opcode, 0 },
{"pwmacccrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMACCCRQ_H, MASK_PWMACCCRQ_H, match_opcode, 0 },
{"pdmacccrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCRQ_B, MASK_PDMACCCRQ_B, match_opcode, 0 },
{"pdmacccrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMACCCRQ_H, MASK_PDMACCCRQ_H, match_opcode, 0 },
{"pwmsubs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBS_B, MASK_PWMSUBS_B, match_opcode, 0 },
{"pwmsubs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBS_H, MASK_PWMSUBS_H, match_opcode, 0 },
{"pwmsubs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBS_W, MASK_PWMSUBS_W, match_opcode, 0 },
{"pdmsubs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBS_B, MASK_PDMSUBS_B, match_opcode, 0 },
{"pdmsubs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBS_H, MASK_PDMSUBS_H, match_opcode, 0 },
{"pdmsubs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBS_W, MASK_PDMSUBS_W, match_opcode, 0 },
{"pwmsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQS_B, MASK_PWMSUBQS_B, match_opcode, 0 },
{"pwmsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQS_H, MASK_PWMSUBQS_H, match_opcode, 0 },
{"pwmsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQS_W, MASK_PWMSUBQS_W, match_opcode, 0 },
{"pdmsubqs.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQS_B, MASK_PDMSUBQS_B, match_opcode, 0 },
{"pdmsubqs.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQS_H, MASK_PDMSUBQS_H, match_opcode, 0 },
{"pdmsubqs.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQS_W, MASK_PDMSUBQS_W, match_opcode, 0 },
{"pwmsubu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBU_B, MASK_PWMSUBU_B, match_opcode, 0 },
{"pwmsubu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBU_H, MASK_PWMSUBU_H, match_opcode, 0 },
{"pwmsubu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBU_W, MASK_PWMSUBU_W, match_opcode, 0 },
{"pdmsubu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBU_B, MASK_PDMSUBU_B, match_opcode, 0 },
{"pdmsubu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBU_H, MASK_PDMSUBU_H, match_opcode, 0 },
{"pdmsubu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBU_W, MASK_PDMSUBU_W, match_opcode, 0 },
{"pwmsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQU_B, MASK_PWMSUBQU_B, match_opcode, 0 },
{"pwmsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQU_H, MASK_PWMSUBQU_H, match_opcode, 0 },
{"pwmsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBQU_W, MASK_PWMSUBQU_W, match_opcode, 0 },
{"pdmsubqu.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQU_B, MASK_PDMSUBQU_B, match_opcode, 0 },
{"pdmsubqu.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQU_H, MASK_PDMSUBQU_H, match_opcode, 0 },
{"pdmsubqu.w", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBQU_W, MASK_PDMSUBQU_W, match_opcode, 0 },
{"pwmsubcc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCC_B, MASK_PWMSUBCC_B, match_opcode, 0 },
{"pwmsubcc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCC_H, MASK_PWMSUBCC_H, match_opcode, 0 },
{"pdmsubcc.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCC_B, MASK_PDMSUBCC_B, match_opcode, 0 },
{"pdmsubcc.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCC_H, MASK_PDMSUBCC_H, match_opcode, 0 },
{"pwmsubccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCCQ_B, MASK_PWMSUBCCQ_B, match_opcode, 0 },
{"pwmsubccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCCQ_H, MASK_PWMSUBCCQ_H, match_opcode, 0 },
{"pdmsubccq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCCQ_B, MASK_PDMSUBCCQ_B, match_opcode, 0 },
{"pdmsubccq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCCQ_H, MASK_PDMSUBCCQ_H, match_opcode, 0 },
{"pwmsubcr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCR_B, MASK_PWMSUBCR_B, match_opcode, 0 },
{"pwmsubcr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCR_H, MASK_PWMSUBCR_H, match_opcode, 0 },
{"pdmsubcr.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCR_B, MASK_PDMSUBCR_B, match_opcode, 0 },
{"pdmsubcr.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCR_H, MASK_PDMSUBCR_H, match_opcode, 0 },
{"pwmsubcrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCRQ_B, MASK_PWMSUBCRQ_B, match_opcode, 0 },
{"pwmsubcrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PWMSUBCRQ_H, MASK_PWMSUBCRQ_H, match_opcode, 0 },
{"pdmsubcrq.b", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCRQ_B, MASK_PDMSUBCRQ_B, match_opcode, 0 },
{"pdmsubcrq.h", 0, INSN_CLASS_XAMETHYST, "d,s,t", MATCH_PDMSUBCRQ_H, MASK_PDMSUBCRQ_H, match_opcode, 0 },

/* Terminate the list.  */
{0, 0, INSN_CLASS_NONE, 0, 0, 0, 0, 0}
};

/* Instruction format for .insn directive.  */
const struct riscv_opcode riscv_insn_types[] =
{
/* name, xlen, isa,          operands, match, mask,    match_func, pinfo.  */
{"r",       0, INSN_CLASS_I,  "O4,F3,F7,d,s,t",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,D,s,t",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,d,S,t",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,D,S,t",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,d,s,T",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,D,s,T",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,d,S,T",     0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F7,D,S,T",     0,    0,  match_opcode, 0 },

{"r",       0, INSN_CLASS_I,  "O4,F3,F2,d,s,t,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,s,t,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,S,t,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,S,t,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,s,T,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,s,T,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,S,T,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,S,T,r",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,s,t,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,s,t,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,S,t,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,S,t,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,s,T,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,s,T,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,d,S,T,R",   0,    0,  match_opcode, 0 },
{"r",       0, INSN_CLASS_F,  "O4,F3,F2,D,S,T,R",   0,    0,  match_opcode, 0 },

{"r4",      0, INSN_CLASS_I,  "O4,F3,F2,d,s,t,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,s,t,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,S,t,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,S,t,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,s,T,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,s,T,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,S,T,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,S,T,r",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,s,t,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,s,t,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,S,t,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,S,t,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,s,T,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,s,T,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,d,S,T,R",   0,    0,  match_opcode, 0 },
{"r4",      0, INSN_CLASS_F,  "O4,F3,F2,D,S,T,R",   0,    0,  match_opcode, 0 },

{"i",       0, INSN_CLASS_I,  "O4,F3,d,s,j",        0,    0,  match_opcode, 0 },
{"i",       0, INSN_CLASS_F,  "O4,F3,D,s,j",        0,    0,  match_opcode, 0 },
{"i",       0, INSN_CLASS_F,  "O4,F3,d,S,j",        0,    0,  match_opcode, 0 },
{"i",       0, INSN_CLASS_F,  "O4,F3,D,S,j",        0,    0,  match_opcode, 0 },

{"i",       0, INSN_CLASS_I,  "O4,F3,d,o(s)",       0,    0,  match_opcode, 0 },
{"i",       0, INSN_CLASS_F,  "O4,F3,D,o(s)",       0,    0,  match_opcode, 0 },

{"s",       0, INSN_CLASS_I,  "O4,F3,t,q(s)",       0,    0,  match_opcode, 0 },
{"s",       0, INSN_CLASS_F,  "O4,F3,T,q(s)",       0,    0,  match_opcode, 0 },

{"sb",      0, INSN_CLASS_I,  "O4,F3,s,t,p",        0,    0,  match_opcode, 0 },
{"sb",      0, INSN_CLASS_F,  "O4,F3,S,t,p",        0,    0,  match_opcode, 0 },
{"sb",      0, INSN_CLASS_F,  "O4,F3,s,T,p",        0,    0,  match_opcode, 0 },
{"sb",      0, INSN_CLASS_F,  "O4,F3,S,T,p",        0,    0,  match_opcode, 0 },

{"b",      0, INSN_CLASS_I,  "O4,F3,s,t,p",        0,    0,  match_opcode, 0 },
{"b",      0, INSN_CLASS_F,  "O4,F3,S,t,p",        0,    0,  match_opcode, 0 },
{"b",      0, INSN_CLASS_F,  "O4,F3,s,T,p",        0,    0,  match_opcode, 0 },
{"b",      0, INSN_CLASS_F,  "O4,F3,S,T,p",        0,    0,  match_opcode, 0 },

{"u",       0, INSN_CLASS_I,  "O4,d,u",             0,    0,  match_opcode, 0 },
{"u",       0, INSN_CLASS_F,  "O4,D,u",             0,    0,  match_opcode, 0 },

{"uj",      0, INSN_CLASS_I,  "O4,d,a",             0,    0,  match_opcode, 0 },
{"uj",      0, INSN_CLASS_F,  "O4,D,a",             0,    0,  match_opcode, 0 },

{"j",      0, INSN_CLASS_I,  "O4,d,a",             0,    0,  match_opcode, 0 },
{"j",      0, INSN_CLASS_F,  "O4,D,a",             0,    0,  match_opcode, 0 },

{"cr",      0, INSN_CLASS_C,  "O2,CF4,d,CV",        0,    0,  match_opcode, 0 },
{"cr",      0, INSN_CLASS_F_AND_C,  "O2,CF4,D,CV",        0,    0,  match_opcode, 0 },
{"cr",      0, INSN_CLASS_F_AND_C,  "O2,CF4,d,CT",        0,    0,  match_opcode, 0 },
{"cr",      0, INSN_CLASS_F_AND_C,  "O2,CF4,D,CT",        0,    0,  match_opcode, 0 },

{"ci",      0, INSN_CLASS_C,  "O2,CF3,d,Co",        0,    0,  match_opcode, 0 },
{"ci",      0, INSN_CLASS_F_AND_C,  "O2,CF3,D,Co",        0,    0,  match_opcode, 0 },

{"ciw",     0, INSN_CLASS_C,  "O2,CF3,Ct,C8",       0,    0,  match_opcode, 0 },
{"ciw",     0, INSN_CLASS_F_AND_C,  "O2,CF3,CD,C8",       0,    0,  match_opcode, 0 },

{"ca",      0, INSN_CLASS_C,  "O2,CF6,CF2,Cs,Ct",   0,    0,  match_opcode, 0 },
{"ca",      0, INSN_CLASS_F_AND_C,  "O2,CF6,CF2,CS,Ct",   0,    0,  match_opcode, 0 },
{"ca",      0, INSN_CLASS_F_AND_C,  "O2,CF6,CF2,Cs,CD",   0,    0,  match_opcode, 0 },
{"ca",      0, INSN_CLASS_F_AND_C,  "O2,CF6,CF2,CS,CD",   0,    0,  match_opcode, 0 },

{"cb",      0, INSN_CLASS_C,  "O2,CF3,Cs,Cp",       0,    0,  match_opcode, 0 },
{"cb",      0, INSN_CLASS_F_AND_C,  "O2,CF3,CS,Cp",       0,    0,  match_opcode, 0 },

{"cj",      0, INSN_CLASS_C,  "O2,CF3,Ca",          0,    0,  match_opcode, 0 },
/* Terminate the list.  */
{0, 0, INSN_CLASS_NONE, 0, 0, 0, 0, 0}
};

/* All standard extensions defined in all supported ISA spec.  */
const struct riscv_ext_version riscv_ext_version_table[] =
{
/* name, ISA spec, major version, minor_version.  */
{"e", ISA_SPEC_CLASS_20191213, 1, 9},
{"e", ISA_SPEC_CLASS_20190608, 1, 9},
{"e", ISA_SPEC_CLASS_2P2,      1, 9},

{"i", ISA_SPEC_CLASS_20191213, 2, 1},
{"i", ISA_SPEC_CLASS_20190608, 2, 1},
{"i", ISA_SPEC_CLASS_2P2,      2, 0},

{"m", ISA_SPEC_CLASS_20191213, 2, 0},
{"m", ISA_SPEC_CLASS_20190608, 2, 0},
{"m", ISA_SPEC_CLASS_2P2,      2, 0},

{"a", ISA_SPEC_CLASS_20191213, 2, 1},
{"a", ISA_SPEC_CLASS_20190608, 2, 0},
{"a", ISA_SPEC_CLASS_2P2,      2, 0},

{"f", ISA_SPEC_CLASS_20191213, 2, 2},
{"f", ISA_SPEC_CLASS_20190608, 2, 2},
{"f", ISA_SPEC_CLASS_2P2,      2, 0},

{"d", ISA_SPEC_CLASS_20191213, 2, 2},
{"d", ISA_SPEC_CLASS_20190608, 2, 2},
{"d", ISA_SPEC_CLASS_2P2,      2, 0},

{"q", ISA_SPEC_CLASS_20191213, 2, 2},
{"q", ISA_SPEC_CLASS_20190608, 2, 2},
{"q", ISA_SPEC_CLASS_2P2,      2, 0},

{"c", ISA_SPEC_CLASS_20191213, 2, 0},
{"c", ISA_SPEC_CLASS_20190608, 2, 0},
{"c", ISA_SPEC_CLASS_2P2,      2, 0},

{"zicsr", ISA_SPEC_CLASS_20191213, 2, 0},
{"zicsr", ISA_SPEC_CLASS_20190608, 2, 0},

{"zifencei", ISA_SPEC_CLASS_20191213, 2, 0},
{"zifencei", ISA_SPEC_CLASS_20190608, 2, 0},

{"zihintpause", ISA_SPEC_CLASS_DRAFT, 1, 0},

{"b",     ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbb",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbs",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zba",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbp",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbe",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbf",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbc",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbr",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbm",   ISA_SPEC_CLASS_DRAFT, 0, 94},
{"zbt",   ISA_SPEC_CLASS_DRAFT, 0, 94},

{"k",     ISA_SPEC_CLASS_DRAFT, 0, 90},
{"zknd",  ISA_SPEC_CLASS_DRAFT, 0, 90},
{"zkne",  ISA_SPEC_CLASS_DRAFT, 0, 90},
{"zknh",  ISA_SPEC_CLASS_DRAFT, 0, 90},
{"zksh",  ISA_SPEC_CLASS_DRAFT, 0, 90},
{"zksed", ISA_SPEC_CLASS_DRAFT, 0, 90},

/* Terminate the list.  */
{NULL, 0, 0, 0}
};

struct isa_spec_t
{
  const char *name;
  enum riscv_isa_spec_class class;
};

/* List for all supported ISA spec versions.  */
static const struct isa_spec_t isa_specs[] =
{
  {"2.2",      ISA_SPEC_CLASS_2P2},
  {"20190608", ISA_SPEC_CLASS_20190608},
  {"20191213", ISA_SPEC_CLASS_20191213},

/* Terminate the list.  */
  {NULL, 0}
};

/* Get the corresponding ISA spec class by giving a ISA spec string.  */

int
riscv_get_isa_spec_class (const char *s,
                         enum riscv_isa_spec_class *class)
{
  const struct isa_spec_t *version;

  if (s == NULL)
    return 0;

  for (version = &isa_specs[0]; version->name != NULL; ++version)
    if (strcmp (version->name, s) == 0)
      {
       *class = version->class;
       return 1;
      }

  /* Can not find the supported ISA spec.  */
  return 0;
}
